UNIT blas;
{ Basic Linear Algebra Subroutines

  small version
  with copy1, axpy, swap, dot, nrm2 and scal                                  }

{$R-}
INTERFACE

TYPE real=double;

{-----------------------------------------------------------------------------}

PROCEDURE Copy1(n : integer; VAR xp, yp);
{ y:=x; x,y - vectors (with increment 1)                                      }

{-----------------------------------------------------------------------------}

PROCEDURE axpy(n : integer; a : real; VAR xp; incx : integer; VAR yp; incy : integer);
{ y:=y + a * x; x,y - vectors (with increment incx, incy), a - scalar         }

{-----------------------------------------------------------------------------}

PROCEDURE Swap(n : integer; VAR xp; incx : integer; VAR yp; incy : integer);
{ x <--> y; x,y - vectors (with increment incx,incy)                          }

{-----------------------------------------------------------------------------}

FUNCTION  dot(n : integer; VAR xp; incx : integer; VAR yp; incy : integer) : real;
{ x^T * y; x,y - vectors (with increment incx,incy)                           }

{-----------------------------------------------------------------------------}

FUNCTION  nrm2(n : integer; VAR xp; incx : integer) : real;
{ (x^T * x)^(0.5); x - vector (with increment x)                              }

{-----------------------------------------------------------------------------}

PROCEDURE scal(n : integer; a : real; VAR xp; incx : integer);
{ x:=a *x; x - vector (with increment incx), a - scalar                       }

{-----------------------------------------------------------------------------}

IMPLEMENTATION

{-----------------------------------------------------------------------------}

PROCEDURE Copy1;
TYPE feldr = ARRAY[1..1] OF real;
VAR      x : feldr ABSOLUTE xp;
         y : feldr ABSOLUTE yp;
         i : integer;
BEGIN
  FOR i := 1 TO n DO
    y[i] := x[i]
END;

{-----------------------------------------------------------------------------}

PROCEDURE axpy;
TYPE feldr = ARRAY[1..1] OF real;
VAR      x : feldr ABSOLUTE xp;
         y : feldr ABSOLUTE yp;
   i,ix,iy : integer;
BEGIN
  ix := 1; iy := 1;
  FOR i := n DOWNTO 1 DO
    BEGIN
      y[iy] := y[iy] + a * x[ix];
      Inc(ix,incx);
      Inc(iy,incy)
    END
END;

{-----------------------------------------------------------------------------}

PROCEDURE Swap;
TYPE feldr = ARRAY[1..1] OF real;
VAR      x : feldr ABSOLUTE xp;
         y : feldr ABSOLUTE yp;
   i,ix,iy : integer;
      temp : real;
BEGIN
  ix := 1; iy := 1;
  FOR i := n DOWNTO 1 DO
    BEGIN
      temp := x[ix]; x[ix] := y[iy]; y[iy] := temp;
      Inc(ix,incx); Inc(iy,incy)
    END
END;

{-----------------------------------------------------------------------------}

FUNCTION dot;
TYPE feldr = ARRAY[1..1] OF real;
VAR      s : real;
         x : feldr ABSOLUTE xp;
         y : feldr ABSOLUTE yp;
 i, ix, iy : integer;
BEGIN
  s  := 0.0;
  ix := 1;
  iy := 1;
  FOR i := 1 TO n DO
    BEGIN
      s := s + x[ix] * y[iy];
      Inc(ix,incx);
      Inc(iy,incy);
    END;
  dot:=s;
END;

{-----------------------------------------------------------------------------}

FUNCTION nrm2;
VAR      x : ARRAY[1..1] OF real ABSOLUTE xp;
  sum,xmax : real;
      ix,i : integer;
BEGIN
  xmax:=0.0;
  ix:=1;
  FOR i:=1 TO n DO
    BEGIN
      IF Abs(x[ix]) > xmax THEN
        xmax:=Abs(x[ix]);
      Inc(ix,incx);
    END;

  IF XMAX <> 0.0 THEN
    BEGIN
      ix:=1;
      sum:=0.0;
      FOR i:=1 TO n DO
        BEGIN
          sum:=sum + Sqr(x[ix]/xmax);
          Inc(ix,incx);
        END;
      nrm2:=xmax*Sqrt(sum);
    END
  ELSE
    nrm2:=0.0;
END;

{-----------------------------------------------------------------------------}

PROCEDURE scal;
VAR   i,ix : integer;
         x : ARRAY[1..1] OF real ABSOLUTE xp;
BEGIN
  ix := 1;
  FOR i := n DOWNTO 1 DO
    BEGIN
      x[ix] := a * x[ix]; Inc(ix,incx)
    END;
END;

{-----------------------------------------------------------------------------}

END.
