{*************************************************************************}
{                                                                         }
{     Turbo Pascal Version 5.0                                            }
{     Interface-Dokumentation zu   D3    3-D-Plot                         }
{                                                                         }
{     Bernd Goldschmidt, Martin-Luther-Universitaet Halle, Biotechnikum   }
{                                                                         }
{     Mit diesem Unit kann man Abhaengigkeiten von zwei Variablen         }
{     graphisch in verschiedenen Formen darstellen.                       }
{     Es unterstuetzt alle Graphikkarten von IBM-kompatiblen Rechnern :   }
{     CGA, EGA, VGA, Hercules                                             }
{                                                                         }
{     Dieses Unit erwartet beim Aufruf, dass                              }
{     1. der gewuenschte Graphik-Modus eingestellt ist                    }
{     2. die Funktionswerte in einem Feld                                 }
{          array[0..Xmax,0..Ymax] of real                                 }
{        uebergeben werden. Dazu muss in einer Variablen vom Typ          }
{          D3PlotArray                                                    }
{        die Adresse des Feldes, die Anzahl der Werte <Xmax,Ymax> sowie   }
{        das Minimum und Maximum der Funktionswerte uebergeben werden.    }
{        Minimum und Maximum koennen mit Hilfe der Procedur               }
{          D3MiniMax                                                      }
{        berechnet werden.                                                }
{                                                                         }
{     Nach der Darstellung springt die Procedur in das aufrufende         }
{     Programm ohne warten zurueck. Dadurch ist es moeglich, mehrere      }
{     Darstellungen gleichzeitig zu erzeugen.                             }
{                                                                         }
{     Die Farben werden durch das Unit beibehalten.                       }
{                                                                         }
{*************************************************************************}

UNIT D3;

INTERFACE

USES
  CRT,Graph;

TYPE real=double;

TYPE D3WindowStyleType  = RECORD
  XBmin   : byte ;     { Bildausschnitt in Prozent }
  YBmin   : byte;
  XBmax   : byte;
  YBmax   : byte;
  Bound   : byte;      { leerer Rand zwischen Bild und Rahmen }
  cls     : boolean ;
  frame   : boolean ;  { Rahmen ja/nein }
  CheckESC: boolean ;
END;

  {  D3WindowStyletype enthaelt die Informationen ueber die Lage der Graphik }

TYPE D3PlotArrayType = RECORD
  AnzPlotX: byte    ; { Anzahl der Spalten im Funktionsfeld }
  AnzPlotY: byte    ; { Anzahl der Zeilen  im Funktionsfeld }
  FktAdr  : pointer ; { Adresse des Funktionsfeldes }
  MaxFunk : real    ; { Minimum und Maximum der Funktionswerte }
  MinFunk : real    ;
END;

   { D3PlotArrayType beschreibt die Funktionswerte }

TYPE D3PersStyleType  = RECORD
  phi     : integer ; { Drehwinkel }
  theta   : integer ; { Aufwinkel  }
  AnzSch  : byte    ; { Anzahl Schichten }
  koord   : boolean ;
  sockel  : boolean ;
END;

    { D3PersStyleType enthaelt die Informationen ueber die perspektivische
       Darstellung der Graphik }

TYPE D3ProjStyleType  = RECORD         { Hoehenlinien }
  AnzH    : byte    ; { Anzahl Hoehenlinien }
  PktRast : boolean ; { Punktraster }
  BezOn   : boolean ; { Bezeichnung }
END;

   { D3ProjStyleType enthaelt die Informationen zur Darstellung
      mit Hoehenlinien }

VAR   D3PerspectiveStyle : D3PersStyleType;
      D3ProjectionStyle  : D3ProjStyleType;
      D3WindowStyle      : D3WindowStyleType;
      D3Error            : shortint;
      D3StandardPlotArray: D3PlotArrayType;

    { in den Variablen
         D3PerspectiveStyle,
         D3ProjectionStyle,
         D3WindowStyle
       werden die aktuellen Daten fuer die Graphik global festgelegt.
       Im Normalfall kann mit der Standardeinstellung gearbeitet werden.
       Jede Funktionswertuebergabe sollte vorher eine Standardzuweisung
       erhalten :
         MeinPlotArray:=D3StandardPlotArray;
       Falls ein Fehler autritt, wird D3Error gesetzt. Im einzelnen
       bedeuten die Fehlernummern :
        1 Nutzerunterbrechung durch ESC
        2 Adresse des Funktionsfeldes nicht uebergeben
        3 Minimum-Maximum nicht berechnet
        4 Anzahl der Funktionswerte nicht uebergeben
        5 Anzahl der Schichten im Schichtplot zu klein
        6 Anzahl der Schichten im Schichtplot zu gross
        7 Anzahl der Hoehenlinien zu klein
     }

PROCEDURE D3MiniMax(VAR    D3PlotArray : D3PlotArrayType);
     { berechnet Minimum und Maximum des Funktionsfeldes }
PROCEDURE D3SCHICHTPLOT(   D3PlotArray : D3PlotArrayType);
     { stellt Schnitte dar }
PROCEDURE D3Hoehe(         D3PlotArray : D3PlotArrayType);
     { Hoehenlinien }
PROCEDURE D3FUNKTIONSPLOT1(D3PlotArray : D3PlotArrayType);
     { perspektivisch, ohne Beruecksichtigung der Sichtbarkeitsverhaeltnisse }
PROCEDURE D3FUNKTIONSPLOT2(D3PlotArray : D3PlotArrayType); { unsichtbar }
     { perspektivisch, mit Beruecksichtigung der Sichtbarkeitsverhaeltnisse }
FUNCTION  D3ErrorMessage(n :shortint): STRING;
     { gibt Fehlermeldung in Abhaengigkeit von der Fehlernummer zurueck }

IMPLEMENTATION

TYPE PlotVector   = ARRAY[0..8000] OF real;

VAR xs,ys,zs,                          { Skalierungskoeffizienten }
    xv,yv,zv,                          { Verschiebung             }
    dr11,dr12,dr13,dr21,dr22,dr23,     { Koeffizienten            }
    xur,yur,zur                : real; { Koordinatenursprung      }
    ZFak,MinFu                 : real;
    KoordOn,SockelOn,
    xinvert,yinvert,zinvert    : boolean;
    Xbreite,Ybreite,
    MaxX,MaxY,MaxF             : integer;
    AnzSchi,AnzHoe,
    MaxPlotX,MaxPlotY          : byte;
    PlotPointer                : ^PlotVector;

FUNCTION ESCTASTE : boolean;
VAR ch : char;
BEGIN
  esctaste:=false;
  IF NOT D3WindowStyle.CheckESC THEN Exit;
  IF KeyPressed THEN BEGIN
    ch:=CRT.ReadKey;esctaste:=(ch=#27);
    IF (ch=#27) THEN D3Error:=1;
  END;
END;

PROCEDURE D3MiniMax(VAR D3PlotArray : D3PlotArrayType);
VAR i          : integer;
    mi,ma,wert : real;
BEGIN
  WITH D3PlotArray DO BEGIN
    PlotPointer:=FktAdr;
    mi:=PlotPointer^[0];ma:=mi;
    FOR i:=0 TO Succ(AnzPlotX)*Succ(AnzPlotY)-1 DO BEGIN
      wert:=PlotPointer^[i];IF mi>wert THEN mi:=wert;IF ma<wert THEN ma:=wert;
    END;
    MinFunk:=mi;MaxFunk:=ma;
  END;
END;

FUNCTION PlotFeld(i,j:byte):real;
   { liefert normierten Funktionswert 0 <= p <= 1 aus dem FktFeld }
BEGIN
  PlotFeld:=(PlotPointer^[i*Succ(MaxPlotY)+j]-MinFu)*ZFak;
END;

FUNCTION InitOK(D3PlotArray : D3PlotArrayType):boolean;
VAR phi1,theta1,dummy:real;

PROCEDURE CleanWindow;
VAR x3,y3,x4,y4 : word;
    vp          : ViewPortType;
BEGIN
  GetViewSettings(vp);
  WITH D3WindowStyle DO BEGIN
    x3:=Trunc(0.01*XBmin*MaxX);x4:=Trunc(0.01*XBmax*MaxX);
    y3:=Trunc(0.01*YBmin*MaxY);y4:=Trunc(0.01*YBmax*MaxY);
  END;
  WITH D3WindowStyle DO BEGIN
    IF cls THEN BEGIN
      SetViewPort(x3,y3,x4,y4,ClipOn);ClearViewPort;
      WITH vp DO SetViewPort(x1,y1,x2,y2,clip);
    END;
    IF frame THEN Rectangle(x3,y3,x4,y4);
  END;
END;

BEGIN
  WITH D3PerspectiveStyle,D3ProjectionStyle,D3PlotArray,D3WindowStyle DO BEGIN
    InitOK:=false;D3Error:=0;
    PlotPointer:=FktAdr;IF FktAdr=NIL THEN BEGIN D3Error:=2;Exit END;
    IF MaxFunk=MinFunk THEN BEGIN D3Error:=3;Exit END;
    MaxPlotX:=AnzPlotX;MaxPlotY:=AnzPlotY;
    IF ((AnzPlotX<1) OR (AnzPlotY<1)) THEN BEGIN D3Error:=4;Exit END;
    MaxX:=GetMaxX;MaxY:=GetMaxY;MaxF:=GetMaxColor;
    IF (cls OR frame) THEN CleanWindow;
    phi1:=phi/180*Pi;theta1:=theta/180*Pi;
    dummy:=Abs(Sin(phi1))+Abs(Cos(phi1));
    Xbreite:=Trunc(0.01*Abs(XBmax-XBmin-2*Bound)*MaxX/dummy);
    dummy:=dummy*Abs(Sin(theta1))+Abs(Cos(theta1));
    Ybreite:=Trunc(0.01*(YBmax-YBmin-2*Bound)*MaxY/dummy);
    dr11:=-Sin(phi1);            dr12:= Cos(phi1);            dr13:= 0;
    dr21:= Cos(phi1)*Sin(theta1);dr22:= Sin(phi1)*Sin(theta1);dr23:=-Cos(theta1);
    xv:=0.5*MaxPlotX;yv:=0.5*MaxPlotY;zv:=0.5;
    xs:=1/MaxPlotX;ys:=1/MaxPlotY;zs:=1;
    xur:=0.005*(XBmin+XBmax)*MaxX;yur:=0.005*(YBmin+YBmax)*MaxY;
    IF Cos(phi1)  <0 THEN xinvert:=true ELSE xinvert:=false;
    IF Sin(phi1)  <0 THEN yinvert:=true ELSE yinvert:=false;
    IF Sin(theta1)<0 THEN zinvert:=true ELSE zinvert:=false;
    ZFak:=1/(MaxFunk-MinFunk);MinFu:=MinFunk;
    AnzSchi:=AnzSch;AnzHoe:=AnzH;
    IF AnzSchi<2        THEN BEGIN D3Error:=5;Exit END;
    IF AnzSchi>MaxPlotX THEN BEGIN D3Error:=6;Exit END;
    IF AnzH<2           THEN BEGIN D3Error:=7;Exit END;
    KoordOn:=koord;SockelOn:=sockel;
    InitOK:=true;
  END;
END;

FUNCTION D3ErrorMessage(n : shortint) : STRING;
BEGIN
  CASE n OF
    1:D3ErrorMessage:='Nutzerunterbrechung';
    2:D3ErrorMessage:='PlotArray-Adresse';
    3:D3ErrorMessage:='min-max nicht definiert';
    4:D3ErrorMessage:='AnzPlot nicht definiert';
    5:D3ErrorMessage:='AnzSchi zu klein';
    6:D3ErrorMessage:='AnzSchi zu gross';
    7:D3ErrorMessage:='AnzH zu klein';
    ELSE D3ErrorMessage:='';
    END;
END;

PROCEDURE SKALIERE(x,y:byte;z:real;VAR xplo,yplo:integer);
VAR xd,yd,zd:real;
BEGIN
  xd:=(x-xv)*xs;yd:=(y-yv)*ys;zd:=(z-zv)*zs;
  xplo:=Trunc(xur+(xd*dr11+yd*dr12)*Xbreite);
  yplo:=Trunc(yur+(xd*dr21+yd*dr22+zd*dr23)*Ybreite);
END;

PROCEDURE ZEICHNESOCKEL;
VAR i,j,k,fa,xa,ya,
  x3,x4,x5,x6,x7,x8,
  y3,y4,y5,y6,y7,y8,
  xmin,xmax,ymin,ymax :integer;
  za,ze               :real;
  color,aktcolor      :word;
  
  FUNCTION sign(x:integer):integer;
  BEGIN IF x<0 THEN sign:=-1 ELSE sign:=1 END;
  
BEGIN
  aktcolor:=GetColor;
  color:=2;IF color>MaxF THEN color:=1;SetColor(color);
  IF xinvert THEN xmin:=MaxPlotX ELSE xmin:=0;xmax:=MaxPlotX-xmin;
  IF yinvert THEN ymin:=MaxPlotY ELSE ymin:=0;ymax:=MaxPlotY-ymin;
  fa:=1;k:=0;
  SKALIERE(xmin,ymax,0,x5,y5);
  za:=PlotFeld(xmin,ymax);SKALIERE(xmin,ymax,za,x6,y6);
  FOR i:=1 TO MaxPlotX DO BEGIN
    IF xinvert THEN xa:=MaxPlotX-i ELSE xa:=i;
    za:=PlotFeld(xa,ymax);
    SKALIERE(xa,ymax,0,x3,y3);
    SKALIERE(xa,ymax,za,x4,y4);
    FOR j:=1 TO Abs(x3-x5)-1 DO BEGIN
      x7:=x5+j*sign(x3-x5);
      y7:=Trunc(y5+(x7-x5)/(x3-x5)*(y3-y5));
      y8:=Trunc(y6+(x7-x5)/(x3-x5)*(y4-y6));
      Line(x7,y7,x7,Succ(y8));k:=Succ(k);
      IF k>4 THEN BEGIN k:=0;SetColor(color) END ELSE SetColor(0);
    END;
    SetColor(color);
    Line(x5,y5,x3,y3);Line(x3,y3,x4,y4);
    Line(x4,y4,x6,y6);Line(x6,y6,x5,y5);
    x5:=x3;x6:=x4;y5:=y3;y6:=y4;
  END;
  SKALIERE(xmax,ymin,0,x5,y5);
  za:=PlotFeld(xmax,ymin);SKALIERE(xmax,ymin,za,x6,y6);
  FOR i:=1 TO MaxPlotY DO BEGIN
    IF yinvert THEN ya:=MaxPlotY-i ELSE ya:=i;
    za:=PlotFeld(xmax,ya);
    SKALIERE(xmax,ya,0,x3,y3);
    SKALIERE(xmax,ya,za,x4,y4);
    FOR j:=1 TO Abs(x3-x5)-1 DO BEGIN
      x7:=x5+j*sign(x3-x5);
      y7:=Trunc(y5+(x7-x5)/(x3-x5)*(y3-y5));
      y8:=Trunc(y6+(x7-x5)/(x3-x5)*(y4-y6));
      Line(x7,y7,x7,Succ(y8));
      IF fa=1 THEN BEGIN fa:=0;SetColor(0) END
      ELSE BEGIN fa:=1;SetColor(color) END;
    END;
    SetColor(color);
    Line(x5,y5,x3,y3);Line(x3,y3,x4,y4);
    Line(x4,y4,x6,y6);Line(x6,y6,x5,y5);
    x5:=x3;x6:=x4;y5:=y3;y6:=y4;
  END;
  SetColor(aktcolor);
END;

PROCEDURE KOORDINATENSYSTEM(hinten : boolean);
VAR x1,x2,x3,x4,x5,x6,x7,x8,
  y1,y2,y3,y4,y5,y6,y7,y8 :integer;
  xmin,ymin,zmin,
  xmax,ymax,zmax          :byte;
  color,aktcolor          :word;
BEGIN
  aktcolor:=GetColor;
  color:=3;IF color>MaxF THEN color:=1;SetColor(color);
  IF xinvert THEN xmin:=0 ELSE xmin:=MaxPlotX;xmax:=MaxPlotX-xmin;
  IF yinvert THEN ymin:=0 ELSE ymin:=MaxPlotY;ymax:=MaxPlotY-ymin;
  IF zinvert THEN zmin:=0 ELSE zmin:=      1;zmax:=1-zmin;
  SKALIERE(xmin,ymin,zmin,x1,y1);
  SKALIERE(xmin,ymin,zmax,x2,y2);
  SKALIERE(xmin,ymax,zmin,x3,y3);
  SKALIERE(xmin,ymax,zmax,x4,y4);
  SKALIERE(xmax,ymin,zmin,x5,y5);
  SKALIERE(xmax,ymin,zmax,x6,y6);
  SKALIERE(xmax,ymax,zmin,x7,y7);
  SKALIERE(xmax,ymax,zmax,x8,y8);
  IF hinten THEN BEGIN
    Line(x3,y3,x7,y7);Line(x7,y7,x5,y5);Line(x2,y2,x4,y4);
    Line(x6,y6,x2,y2);Line(x5,y5,x6,y6);Line(x3,y3,x4,y4);
    Line(x7,y7,x8,y8);Line(x4,y4,x8,y8);Line(x8,y8,x6,y6);
    Skaliere(0,0,0,x1,y1)      ;OutTextXY(x1,y1,'0');
    Skaliere(MaxPlotX,0,0,x1,y1);OutTextXY(x1,y1,'X');
    Skaliere(0,MaxPlotY,0,x1,y1);OutTextXY(x1,y1,'Y');
    Skaliere(0,0,1,x1,y1)      ;OutTextXY(x1,y1,'Z');
  END
  ELSE BEGIN
    Line(x1,y1,x3,y3);Line(x5,y5,x1,y1);Line(x1,y1,x2,y2);
  END;
  SetColor(aktcolor);
END;

PROCEDURE D3SCHICHTPLOT(D3PlotArray : D3PlotArrayType);
VAR x,i   : byte;
    diff  : real;
    Color : word;

  PROCEDURE SCHICHT(x:byte);
  VAR i,j,k,fa,
  x3,x4,x5,x6,x7,x8,
  y3,y4,y5,y6,y7,y8 :integer;
  ii                :byte;
  za                :real;
  
  FUNCTION sign(x:integer):integer;
  BEGIN IF x<0 THEN sign:=-1 ELSE sign:=1 END;
  
  BEGIN
    Inc(Color);IF Color>MaxF THEN Color:=1;
    SetColor(Color);
    fa:=Color;k:=0;
    SKALIERE(x,0,0,x5,y5);
    za:=PlotFeld(x,0);SKALIERE(x,0,za,x6,y6);
    FOR ii:=1 TO MaxPlotY DO BEGIN
      IF yinvert THEN i:=MaxPlotY-ii ELSE i:=ii;
      za:=PlotFeld(x,i);
      SKALIERE(x,i,0,x3,y3);
      SKALIERE(x,i,za,x4,y4);
      FOR j:=1 TO Abs(x3-x5)-1 DO BEGIN
        x7:=x5+j*sign(x3-x5);
        y7:=Trunc(y5+(x7-x5)/(x3-x5)*(y3-y5));
        y8:=Trunc(y6+(x7-x5)/(x3-x5)*(y4-y6));
        Line(x7,y7,x7,Succ(y8));
        IF fa=Color THEN BEGIN fa:=0    ;SetColor(fa) END
        ELSE BEGIN fa:=Color;SetColor(fa) END;
      END;
      SetColor(0);Line(x6,y6-1,x4,y4-1);
      SetColor(Color);
      Line(x5,y5,x3,y3);Line(x3,y3,x4,y4);
      Line(x4,y4,x6,y6);Line(x4+1,y4,x6+1,y6);
      Line(x6,y6,x5,y5);
      x5:=x3;x6:=x4;y5:=y3;y6:=y4;
    END;
    Line(x4+1,y4,x4+1,y3);
    SetColor(0);
    Line(x4+2,y4,x4+2,y3);Line(x4+3,y4,x4+3,y3);
    SetColor(Color);
  END; { schicht }
  
BEGIN
  IF NOT InitOK(D3PlotArray) THEN Exit;
  Color:=0;
  diff:=Succ(MaxPlotX)/Pred(AnzSchi);
  IF KoordOn THEN KOORDINATENSYSTEM(true);
  FOR i:=1 TO AnzSchi DO BEGIN
    x:=Trunc((i-1)*diff);
    IF xinvert THEN x:=MaxPlotX-x;
    SCHICHT(x);
    IF ESCTaste THEN Exit;
  END;
  IF KoordOn THEN KOORDINATENSYSTEM(false);
END;{ d3schichtplot }

PROCEDURE D3Hoehe(D3PlotArray : D3PlotArrayType);
VAR XScalHoehe,
  YScalHoehe : real;
  i,j,k      : byte;
  x1,x2,y1,y2: integer;
  Color      : word;
  
  FUNCTION ScalX(i:real):integer;
  BEGIN
    scalx:=Trunc(x1+i*XScalHoehe);
  END;
  
  FUNCTION ScalY(i:real):integer;
  BEGIN
    scaly:=Trunc(y2-i*YScalHoehe);
  END;
  
  PROCEDURE ZeichneHoehenLinie(h:real);
  VAR i,j,Stru : byte;
  
  FUNCTION Inter(x1,x2:real):real;
  BEGIN
    Inter:=Abs(x1/(x1-x2));
  END;
  
  FUNCTION IX(i,j:byte;h:real):integer;
  BEGIN
    IX:=ScalX(j+Inter(PlotFeld(j,i)-h,PlotFeld(Succ(j),i)-h));
  END;
  
  FUNCTION IY(i,j:byte;h:real):integer;
  BEGIN
    IY:=ScalY(i+Inter(PlotFeld(j,i)-h,PlotFeld(j,Succ(i))-h));
  END;
  
BEGIN
  Inc(Color);IF Color>MaxF THEN Color:=1;
  SetColor(Color);
  FOR i:=1 TO MaxPlotY DO BEGIN
    IF PlotFeld(0,Pred(i))<h THEN Stru:=0 ELSE Stru:=1;      {  2 --- 8 }
    IF PlotFeld(0,i)>=h THEN Inc(Stru,2);                    {  !     ! }
    FOR j:=1 TO MaxPlotX DO BEGIN                            {  1 --- 4 }
      IF PlotFeld(j,Pred(i))>=h THEN Inc(Stru,4);
      IF PlotFeld(j,i)>=h THEN Inc(Stru,8);
      CASE Stru OF
        0,15:;
        1,14:Line(ScalX(Pred(j)),IY(Pred(i),Pred(j),h),IX(Pred(i),Pred(j),h),ScalY(Pred(i)));
        2,13:Line(ScalX(Pred(j)),IY(Pred(i),Pred(j),h),IX(i,Pred(j),h),ScalY(i));
        3,12:Line(IX(Pred(i),Pred(j),h),ScalY(Pred(i)),IX(i,Pred(j),h),ScalY(i));
        4,11:Line(IX(Pred(i),Pred(j),h),ScalY(Pred(i)),ScalX(j),IY(Pred(i),j,h));
        5,10:Line(ScalX(Pred(j)),IY(Pred(i),Pred(j),h),ScalX(j),IY(Pred(i),j,h));
        7,8 :Line(IX(i,Pred(j),h),ScalY(i),ScalX(j),IY(Pred(i),j,h));
        6   :BEGIN
          Line(ScalX(Pred(j)),IY(Pred(i),Pred(j),h),IX(Pred(i),Pred(j),h),ScalY(Pred(i)));
          Line(IX(i,Pred(j),h),ScalY(i),ScalX(j),IY(Pred(i),j,h));
        END;
        9   :BEGIN
          Line(ScalX(Pred(j)),IY(Pred(i),Pred(j),h),IX(i,Pred(j),h),ScalY(i));
          Line(IX(Pred(i),Pred(j),h),ScalY(Pred(i)),ScalX(j),IY(Pred(i),j,h));
        END;
      END;
      Stru:=Stru DIV 4;
    END;
  END;
END;

BEGIN
  IF NOT InitOK(D3PlotArray) THEN Exit;
  WITH D3WindowStyle DO BEGIN
    x1:=Trunc(0.01*(XBmin+Bound)*MaxX);
    x2:=Trunc(0.01*(XBmax-Bound)*MaxX);
    y1:=Trunc(0.01*(YBmin+Bound)*MaxY);
    y2:=Trunc(0.01*(YBmax-Bound)*MaxY);
  END;
  XScalHoehe:=(x2-x1)/MaxPlotX;YScalHoehe:=(y2-y1)/MaxPlotY;
  Rectangle(x1,y1,x2,y2);
  WITH D3ProjectionStyle DO BEGIN
    IF PktRast THEN FOR i:=0 TO MaxPlotX DO FOR j:=0 TO MaxPlotY DO
      PutPixel(ScalX(i),ScalY(j),1);
    IF BezOn THEN BEGIN
      OutTextXY(x1-8,y2,'0');OutTextXY(x1-8,y1,'Y');OutTextXY(x2-8,y2,'X');
     END;
  END;
  Color:=0;
  FOR k:=1 TO Pred(AnzHoe) DO BEGIN
    ZeichneHoehenlinie(k/AnzHoe);IF ESCTaste THEN Exit;
  END;
END;  { d3hoehe }

PROCEDURE D3FUNKTIONSPLOT1(D3PlotArray : D3PlotArrayType); { sichtbar }
VAR x1,y1,x2,y2    :integer;
    i,ii,j,jj      :byte;
    x,y,z          :real;
    ch             :char;
BEGIN
  IF NOT InitOK(D3PlotArray) THEN Exit;
  IF KoordOn THEN KoordinatenSystem(true);
  FOR i:=MaxPlotY DOWNTO 0 DO BEGIN
    IF xinvert THEN jj:=0 ELSE jj:=MaxPlotX;
    IF yinvert THEN ii:=i ELSE ii:=MaxPlotY-i;
    SKALIERE(jj,ii,PlotFeld(jj,ii),x1,y1);
    FOR j:=MaxPlotX-1 DOWNTO 0 DO BEGIN
      IF xinvert THEN jj:=MaxPlotX-j ELSE jj:=j;
      SKALIERE(jj,ii,PlotFeld(jj,ii),x2,y2);
      Line(x1,y1,x2,y2);
      x1:=x2;y1:=y2;
    END;
    IF ESCTaste THEN Exit;
  END;
  FOR i:=MaxPlotX DOWNTO 0 DO BEGIN
    IF xinvert THEN ii:=i ELSE ii:=MaxPlotX-i;
    IF yinvert THEN jj:=0 ELSE jj:=MaxPlotY;
    SKALIERE(ii,jj,PlotFeld(ii,jj),x1,y1);
    FOR j:=MaxPlotY-1 DOWNTO 0 DO BEGIN
      IF yinvert THEN jj:=MaxPlotY-j ELSE jj:=j;
      SKALIERE(ii,jj,PlotFeld(ii,jj),x2,y2);
      Line(x1,y1,x2,y2);
      x1:=x2;y1:=y2;
    END;
    IF ESCTaste THEN Exit;
  END;
  IF SockelOn THEN ZeichneSockel;
  IF KoordOn THEN KoordinatenSystem(false);
END; { d3funktionsplot1 }

PROCEDURE D3FUNKTIONSPLOT2(D3PlotArray : D3PlotArrayType);
TYPE RealPointType = RECORD x,y : real END;
VAR k,i,j,ii,jj :integer;
    pp          :ARRAY[0..4] OF PointType;
BEGIN
  IF NOT InitOK(D3PlotArray)THEN Exit;
  IF KoordOn THEN KoordinatenSystem(true);
  FOR ii:=1 TO MaxPlotX DO FOR jj:=1 TO MaxPlotY DO BEGIN
    IF xinvert THEN i:=MaxPlotX+1-ii ELSE i:=ii;
    IF yinvert THEN j:=MaxPlotY+1-jj ELSE j:=jj;
    SKALIERE(i,j,PlotFeld(i,j),pp[1].x,pp[1].y);
    SKALIERE(Pred(i),j,PlotFeld(Pred(i),j),pp[2].x,pp[2].y);
    SKALIERE(Pred(i),Pred(j),PlotFeld(Pred(i),Pred(j)),pp[3].x,pp[3].y);
    SKALIERE(i,Pred(j),PlotFeld(i,Pred(j)),pp[4].x,pp[4].y);
    pp[0]:=pp[4];
    SetFillStyle(Solidfill,0); FillPoly(5,pp);
    IF ESCTaste THEN Exit;
  END;
  IF SockelOn THEN ZeichneSockel;
  IF KoordOn THEN KoordinatenSystem(false);
END; { D3Funktionsplot2 }

BEGIN
  WITH D3StandardPlotArray DO BEGIN
    AnzPlotX:=0;  AnzPlotY:=0;  FktAdr :=NIL;
    MaxFunk :=0;  MinFunk :=0;
  END;
  WITH D3WindowStyle DO BEGIN
    XBmin   :=  5;    YBmin   :=  5;
    XBmax   := 95;    YBmax   := 90;
    Bound   :=  2;
    cls     := true; frame    := true; CheckESC  := true;
  END;
  WITH D3PerspectiveStyle DO BEGIN
    phi     := 30;    theta   := 30;
    AnzSch  := 10;
    koord   := true;  sockel  := true;
  END;
  WITH D3ProjectionStyle DO BEGIN
    AnzH := 20; PktRast := true; BezOn := true;
  END;
END.
