UNIT def;
{ definitions of constants, types and global variables }

INTERFACE

USES Graph;    { for menu- and backgroundcolor }

TYPE real=double;

CONST { The following constants can be changed in the program }
      BGI     : STRING = '';      { path to *.bgi                            }
      EPS0    : real = 1.0E-10;   { bound for defect of | F |                }
      EPS1    : real = 1.0E-6;    { bound for relative change in x^k         }
      EPS3    : real = 1.0E-10;   { bound for relative change in || F^k ||   }
      EPSGrad : real = 1.0E-10;   { bound for Gradient | J^T F |             }
      EPSDesc : real = 1.0E-10;   { bound for F^T * J * p                    }
      STEPMAX : integer = 100;    { maximal number of Gauss-Newton steps     }
      No_Intervals : integer = 19;{ No. of Intervals for 3-D plot            }

      EPS2    = 1.0E-3;           { test for relative change in x if x=0     }
      EPSDiff1= 1.0E-08;          { for numerical differentiation of F       }
      EPSDiff2= 1.0E-10;          { for numerical differentiation of F       }

      NBANDS  =  6;               { band width of the leastsquarematrix      }
                                  { = maximal order of B-Splines             }
      NMAX    = 35;               { maximal number of B-Splines              }
      MMAX    = 160;              { maximal number of datavalues             }
      NKMAX   = NMAX+NBANDS;      { NMAX+NBANDS                              }
      MAXNCSTR= 2*NMAX;           { maximal number of constraints            }
      LMAX    = NMAX;             { maximal number of free knots             }


      { The following three constants are the menu colors }
      MyBackground= LightGray;
      MyMenuBackground= Blue;
      MyMenuForeground= White;

TYPE dataabscissae      = ARRAY [1..MMAX] OF real;
     datavalues         = ARRAY [1..MMAX] OF real;
     knotsequence       = ARRAY [1..NKMAX] OF real;
     free_knots         = ARRAY [1..NKMAX] OF integer;
     coefficients       = ARRAY [1..NMAX] OF real;
     MatrixNxN          = ARRAY [1..NMAX,1..NMAX] OF real;
     RowVector          = ARRAY [1..NBANDS] OF real;
     LeastSquareMatrix  = ARRAY [1..NMAX,1..NBANDS] OF real;
     SmoothingMatrix    = ARRAY [1..NMAX,1..NBANDS] OF real;
     SmoothingVector    = ARRAY [1..MMAX+NMAX] OF real;
     OrthogonalMatrix   = ARRAY [1..MMAX,1..NBANDS] OF real;
     JacobiMatrix       = ARRAY [1..MMAX,1..LMAX] OF real;
     SmoothJacobiMatrix = ARRAY [1..MMAX+NMAX,1..LMAX] OF real;
     CSTRMatrix         = ARRAY [1..MAXNCSTR,1..LMAX] OF real;
     CSTRVector         = ARRAY [1..MAXNCSTR] OF real;
     PlotArrayD3Type    = ARRAY [0..0] OF real;
     PlotArrayD3Pointer = ^PlotArrayD3Type;

VAR m,                          { number of data values (tau[i],y[i]) i=1..m }
    order,                      { order of B-Splines                         }
    l,                          { number of free knots                       }
    r,                          { order of derivative in smoothing term      }
    step,                       { number of Gauss-Newton Steps               }
    n         : integer;        { number of B-Splines                        }

    t         : knotsequence;   { vector of spline knots                     }
    per       : free_knots;     { pointer to free knots                      }
    tau       : dataabscissae;
    y         : datavalues;
    alpha     : coefficients;   { unknown spline coefficients                }

    Kaufman_Model,              { Kaufman Model or Golub/Pereyra Model       }
    out_of_interval,            { actual point outside spline interval       }
    Constraints_abs,            { relative or absolute constraints for knots }
    How       : Boolean;        { Smoothing spline or Least Squares Spline   }

    ProgramNo,                  { Selected Menu Item                         }
    left,                       { no. of spline knot left from actual point  }
    RetCode   : integer;        { Return Code from Gauss-Newton methods      }

    mu,                         { Smoothing parameter                        }
    RES,                        { Residual sum                               }
    Min_Dist_Knots,             { Minimum Distance between knots             }
    normGradient,               { Norm of Gradient                           }
    grad_p,                     { Gradient^T * Search direction              }
    macheps,                    { machine epsilon                            }
    Start,
    Stop      : real;           { Timing variables                           }
    TypeOfConstraints : char;   { Type of constraints on derivatives         }

IMPLEMENTATION
BEGIN
  macheps:=2.220446049E-16; { double t=53 ==> 2^(-(t-1) }
  m:=MMAX;
  n:=NMAX;
  order:=4;
  l:=0;
  r:=2;
  mu:=0.0;
  TypeOfConstraints:='N';
END. { def }