UNIT GIVENS;
{ low level procedures for GIVENS plane rotations                             }
{$R-}
INTERFACE
    USES math;    { low level mathematical subroutines                        }

TYPE real=double;

VAR CosG, SinG: real;       { parameter for GIVENS plane rotation }

{-----------------------------------------------------------------------------}

PROCEDURE setup_GIVENS_rotation( a, b: real; VAR xi : real );
{ setup GIVENS plane rotation
  The global varaibles CosG and SinG are computed so that the application of
  the rotation annihilates b.
  The parameter xi can be used for latter reconstruction of the rotation      }

{-----------------------------------------------------------------------------}

PROCEDURE reconstruct_GIVENS_rotation ( xi : real );
{ reconstruct parameter CosG and SinG for GIVENS plane rotation               }

{-----------------------------------------------------------------------------}

PROCEDURE apply_GIVENS_rotation(     n: integer;
                                 VAR addr_x,addr_y {: array[ 1: n] of real }
                                );
{ apply GIVENS plane rotation the parameters of which are stored in CosG and
  SinG                                                                        }

{-----------------------------------------------------------------------------}

IMPLEMENTATION

{-----------------------------------------------------------------------------}

PROCEDURE setup_GIVENS_rotation ( a,b: real; VAR xi : real );

VAR roe,scale,r                  : real;

BEGIN
  IF b = 0.0 THEN
    BEGIN
      CosG:=1.0;
      SinG:=0.0;
      r:=0.0;
      Exit;
    END;

  IF Abs( a) > Abs( b) THEN
    roe := a
  ELSE roe := b;
  scale := Abs( a) + Abs( b);
  IF scale = 0.0 THEN
    BEGIN
      CosG := 1.0;
      SinG := 0.0;
      r   := 0.0;
    END
  ELSE
    BEGIN
      r := sign( roe) * scale * Sqrt( Sqr( a/scale) + Sqr( b/scale));
      CosG := a/r;
      SinG := b/r
    END{ if };

  { for the reconstruction of the parameters
    without directly overwriting a or b       }

  xi:=1.0;
  IF Abs(a) > Abs(b) THEN xi:=SinG
  ELSE IF CosG<> 0.0 THEN xi:=1.0/CosG;

END; { setup_GIVENS_rotation }

{-----------------------------------------------------------------------------}

PROCEDURE reconstruct_GIVENS_rotation ( xi : real );

BEGIN
  IF xi = 1.0 THEN
    BEGIN
      CosG:=0.0;
      SinG:=1.0;
    END
  ELSE
    IF Abs(xi) < 1.0 THEN
      BEGIN
        CosG:=Sqrt(1.0-Sqr(xi));
        SinG:=xi;
      END
    ELSE
      BEGIN
        CosG:=1.0/xi;
        SinG:=Sqrt(1.0-Sqr(CosG));
      END;
END; { reconstruct_GIVENS_rotation }

{-----------------------------------------------------------------------------}

PROCEDURE apply_GIVENS_rotation(     n: integer;
                                 VAR addr_x,addr_y {: array[ 1: n] of real }
                                );

TYPE vector = ARRAY[ 1..1] OF real;

VAR x                   : vector ABSOLUTE addr_x;
    y                   : vector ABSOLUTE addr_y;
    temp                : real;
    i                   : integer;

BEGIN
  {$R-}

  IF (n = 0) OR (CosG=1.0) THEN Exit;

  FOR i := 1 TO n DO
    BEGIN
      temp := CosG * x[i] + SinG * y[i];
      y[i] := -SinG * x[i] + CosG * y[i];
      x[i] := temp;
    END{ for i }
END; { apply _GIVENS_rotation }

{-----------------------------------------------------------------------------}

END.{ Unit GIVENS }
