UNIT Graphic;
{ with special thanks to Volker Kunert, FB Mathematik und Informatik for
  Metafile Output to TeX and to Prof.B.Goldschmidt for Unit D3                }


INTERFACE

USES  def,        { definitions of constants, types and global variables      }
      error,      { error handler                                             }
      various,    { various procedures for menu handling                      }
      BSpline,    { procedures to compute B-Splines and l2-approximations     }
      smooth,     { procedures to compute smoothing B-Splines                 }
      d3,         { Three dimensional representation of residual sum          }
      crt, graph, printer, dos; { Standard Turbo-Pascal Units                 }

PROCEDURE graphik;

IMPLEMENTATION

CONST CoordX=35;
      CoordY=10;
      TeX_MaxX   : integer = maxint;
      TeX_MaxY   : integer = maxint;
      TeX_CoordX : integer = 1767;
      TeX_CoordY : integer = 1767;
      TeX_MinX = 0;
      TeX_MinY = 0;

TYPE workstring=STRING[20];

VAR Xmin,Xmax,YminGlb,YmaxGlb,YMax,Ymin,Xorig,Yorig,fak,
    xfak,yfak                                                 :real;
    Xpos,Ypos,dXCoord,dYCoord,k,left_min,left_max,i,XExponent,
    YExponent,TeX_dXCoord,TeX_dYCoord       	              :integer;
    Year,Month,Day,DayofWeek                                  :word;
    ch                                                        :workstring;
    character                                                 :char;
    Flag,Hardcopy,TeX_Output                                  :Boolean;
    HeadLine                                                  :STRING[60];
    TeX_File_Name                                             :STRING[20];
    TeX_File                                                  :text;
    MyPlotArray                                               :D3PlotArrayType;
    FileOutput                                                :char;

{-----------------------------------------------------------------------------}

PROCEDURE InitGr;
VAR PathToDriver : STRING;  { Stores the DOS path to *.BGI & *.CHR }
    GraphDriver,GraphMode,ErrorCode :integer;
BEGIN
  PathToDriver :=BGI;
  REPEAT

    GraphDriver := Detect;                { use autodetection }

    InitGraph(GraphDriver, GraphMode, PathToDriver);
    ErrorCode := GraphResult;             { preserve error return }
    IF ErrorCode <> grOK THEN             { error? }
      BEGIN
        WriteLn;
        WriteLn('Graphics error: ', GraphErrorMsg(ErrorCode));
        IF ErrorCode = grFileNotFound THEN  { Can't find driver file }
          BEGIN
            WriteLn('Enter full path to BGI driver or type <Ctrl-Break> to quit:');
            ReadLn(PathToDriver);
            WriteLn;
          END
        ELSE
          Error_Handler('InitGr: Could not initialize graphic');
        { Some other error: terminate }
      END;
    BGI:=PathToDriver;
  UNTIL ErrorCode = grOK;
END; { InitGr }

{-----------------------------------------------------------------------------}

PROCEDURE SetWorldCoord;
BEGIN
  dXCoord:=GetMaxX-CoordX-5;dYCoord:=GetMaxY-CoordY-10;
END;
{-----------------------------------------------------------------------------}

FUNCTION ScalX(x:real):integer;
VAR du:real;
BEGIN
  du:=dXCoord*(x-XMin)/(Xmax-Xmin);
  ScalX:=CoordX+Trunc(du);
END;
{-----------------------------------------------------------------------------}

FUNCTION ScalY(y:real):integer;
VAR du:real;
BEGIN
  du:=dYCoord*(y-YMinGlb)/(YMaxGlb-YminGlb);
  ScalY:=CoordY+dYCoord-Trunc(du);
END;
{-----------------------------------------------------------------------------}

PROCEDURE ScalMax;
BEGIN
  fak:=Abs(Xmin);
  IF fak<Abs(Xmax) THEN fak:=Abs(Xmax);
  xfak:=1;
  XExponent:=0;
  YExponent:=0;
  WHILE fak<0.5 DO
    BEGIN
      fak:=fak*10;xfak:=xfak*10;
      XExponent:=XExponent-1;
    END;
  WHILE fak>1000 DO
    BEGIN
      fak:=fak*0.1;xfak:=xfak*0.1;
      XExponent:=XExponent+1;
    END;
  fak:=Abs(YminGlb);
  IF fak<Abs(YmaxGlb) THEN fak:=Abs(YmaxGlb);
  yfak:=1;
  WHILE fak<0.5 DO
    BEGIN
      fak:=fak*10;yfak:=yfak*10;
      YExponent:=YExponent-1;
    END;
  WHILE fak>1000 DO
    BEGIN
      fak:=fak*0.1;yfak:=yfak*0.1;
      YExponent:=YExponent+1;
    END;
END;
{-----------------------------------------------------------------------------}

PROCEDURE DrawNum(x:real;length_st,xpos,ypos:integer);
VAR st:workstring;
BEGIN
  x:=x*fak;Str(x:20:10,st);
  WHILE Pos(' ',st)>0 DO Delete(st,Pos(' ',st),1);
  Delete(st,length_st+1,20);
  OutTextXY(xpos,ypos,st);
END;
{-----------------------------------------------------------------------------}

PROCEDURE DrawAxis(plot_derivative_no:integer);
VAR XSet,Yset:real;
    k:integer;
BEGIN
  Rectangle(CoordX,CoordY,CoordX+dXCoord,CoordY+dYCoord);
  ScalMax;
  fak:=xfak;
  FOR k:=1 TO 9 DO
    BEGIN
      Xset:=Xmin+(Xmax-Xmin)/10*k;
      Line(ScalX(XSet),CoordY+dYCoord-2,ScalX(Xset),CoordY+DYCoord+2);
      xpos:=ScalX(XSet)-17;ypos:=CoordY+dYCoord+2;
      DrawNum(Xset,4,xpos,ypos);
    END;
  fak:=Abs(xmin);
  IF Abs(xmax)>fak THEN fak:=Abs(xmax);
  Str(XExponent:3,ch);
  ch:='E'+ch;
  OutTextXY(CoordX+dXCoord-40,Coordy+dYCoord-8,ch);
  fak:=yfak;
  FOR k:=1 TO 9 DO
    BEGIN
      Yset:=YminGlb+(YmaxGlb-YminGlb)/10*k;
      Line(CoordX-2,ScalY(Yset),CoordX+2,ScalY(Yset));
      xpos:=0;ypos:=ScalY(Yset)-3;
      DrawNum(Yset,4,xpos,ypos);
    END;
  fak:=Abs(YminGlb);
  IF Abs(YmaxGlb)>fak THEN fak:=Abs(YmaxGlb);
  Str(YExponent:3,ch);
  ch:='E'+ch;
  OutTextXY(CoordX+5,Coordy+2,ch);
  IF plot_derivative_no=0 THEN
    BEGIN
      FOR k:=1 TO m DO
        IF (Xmin<=tau[k]) AND (tau[k]<=Xmax) THEN
          Circle(ScalX(tau[k]),ScalY(y[k]),3);
    END;
  FOR k:=1 TO n+order DO
    IF (Xmin<=t[k]) AND (t[k]<=Xmax) THEN
      Bar(ScalX(t[k])-1,CoordY+dYCoord-6,ScalX(t[k])+1,CoordY+dYCoord+2);
END;  { DrawAxis }
{-----------------------------------------------------------------------------}

PROCEDURE Switch_TeX (s: Boolean);       { TeX Graphic on/off              }
BEGIN
  TeX_Output:=s;
  {$i-}
  Flush(TeX_File);
  {$i+}
  IF IOResult<>0 THEN {do nothing};
END;
{-----------------------------------------------------------------------------}

PROCEDURE TeX_Scale (xmax,ymax:integer);
BEGIN
  WriteLn(TeX_File,'\begingroup\normalsize\unitlength=10cm');
  WriteLn(TeX_File,'\divide \unitlength by ',ymax);
  WriteLn(TeX_File,'\def\lineto(#1,#2){\put(#1,#2){\special{em:lineto}}}');
  WriteLn(TeX_File,'\def\moveto(#1,#2){\put(#1,#2){\special{em:moveto}}}');
  WriteLn(TeX_File,'\def\linewidth(#1){\special{em:linewidth #1}}');
  WriteLn(TeX_File,'\def\numbersize{\footnotesize}');
  WriteLn(TeX_File,'\def\overtext{}','\def\undertext{\vss}');
  WriteLn(TeX_File,'\def\lefttext{}','\def\righttext{\hss}');
  WriteLn(TeX_File,'\begin{picture}(',xmax,',',ymax,')');
END;
{-----------------------------------------------------------------------------}

PROCEDURE TeX_Init;
CONST TeX_File_Number:integer=0;
VAR s:STRING[8];
BEGIN
  Inc(TeX_File_Number);
  Str( TeX_File_Number, s);
  TeX_File_Name:=s+'.TeX';
  Assign(TeX_File, TeX_File_name);
  {$I-}
  Rewrite(TeX_File);
  {$I+}
  IF IOResult<>0 THEN
    BEGIN
      WriteLn('Error in TeX.TeX_Init.83');
      Exit;
    END;

  WriteLn(TeX_File,'%%%%%%%%%%% Picture Nr.',TeX_File_Number,' %%%%%%%%%');
  WriteLn(TeX_File);
  TeX_Scale( TeX_MaxX,TeX_MaxY);
END;
{-----------------------------------------------------------------------------}

PROCEDURE TeX_Initgraph;
BEGIN
  Switch_TeX( true);
  TeX_Init;
END;
{-----------------------------------------------------------------------------}

PROCEDURE TeX_SetWorldCoord;
BEGIN
  TeX_dXCoord:=TeX_MaxX-TeX_CoordX-1000;
  TeX_dYCoord:=TeX_MaxY-TeX_CoordY-1000;
END;
{-----------------------------------------------------------------------------}

PROCEDURE TeX_Moveto (x,y:integer);
BEGIN
  WriteLn(TeX_File,'\moveto(',x,',',y,')');
END;
{-----------------------------------------------------------------------------}

PROCEDURE TeX_Lineto (x,y: integer);
BEGIN
  WriteLn(TeX_File,'\lineto(',x,',',y,')');
END;
{-----------------------------------------------------------------------------}

PROCEDURE TeX_Set_Linewidth(linewidth:STRING);
BEGIN
  IF linewidth='' THEN linewidth:='1pt';
  WriteLn(TeX_File,'\linewidth(',linewidth, ')');
END;
{-----------------------------------------------------------------------------}

PROCEDURE TeX_Def( x, y: STRING);
BEGIN
  WriteLn(TeX_File,'\def',x,'{',y,'}');
END;
{-----------------------------------------------------------------------------}

PROCEDURE TeX_SetTextjustify (horiz,vert: word);
BEGIN
  { lefttext means reference point is left of text }
  IF (horiz=centertext) OR (horiz=righttext) THEN
    TeX_Def('\lefttext','\hss')
  ELSE TeX_Def('\lefttext','');
  IF (horiz=centertext) OR (horiz=lefttext) THEN
    TeX_Def('\righttext','\hss')
  ELSE TeX_Def('\righttext','');
  IF (vert=Centertext) OR (vert=bottomtext) THEN
    TeX_Def('\overtext','\vss')
  ELSE TeX_Def('\overtext','');
  IF (vert=Centertext) OR (vert=toptext) THEN
    TeX_Def('\undertext','\vss')
  ELSE TeX_Def('\undertext','');
END;
{-----------------------------------------------------------------------------}

PROCEDURE TeX_OutTextXY (x,y: integer;s: STRING);
BEGIN
  WriteLn(TeX_File,'\put(',x,',',y,'){\vbox to0pt{\overtext',
  '\hbox to0pt{\lefttext \numbersize ',s,'\righttext}\undertext}}');
END;
{-----------------------------------------------------------------------------}

PROCEDURE TeX_Line( x1,y1,x2,y2: integer);
BEGIN
  TeX_Moveto(x1,y1); TeX_Lineto( x2, y2);
END;
{-----------------------------------------------------------------------------}

PROCEDURE TeX_PutPixel( xpos,ypos: integer; Color: byte);
BEGIN
  TeX_OuttextXY( xpos, ypos,'.')
END;
{-----------------------------------------------------------------------------}

PROCEDURE TeX_Circle (x,y: integer;Radius: word);
BEGIN
  WriteLn(TeX_File,'\put(',x,',',y,'){\circle{',Radius,'}}');
END;
{-----------------------------------------------------------------------------}

PROCEDURE TeX_Rectangle (x1,y1,x2,y2: integer);
BEGIN
  TeX_Moveto(x1,y1);
  TeX_Lineto( x2, y1);
  TeX_Lineto( x2, y2);
  TeX_Lineto( x1, y2);
  TeX_Lineto( x1, y1);
END;
{-----------------------------------------------------------------------------}

FUNCTION  TeX_ScalX (x: real): integer;
VAR du : real;
BEGIN
  du:=TeX_dXcoord*(x-Xmin)/(Xmax-Xmin);
  TeX_ScalX:=TeX_CoordX+Trunc(du);
END;
{-----------------------------------------------------------------------------}

FUNCTION  TeX_ScalY (y: real): integer;
VAR du: real;
BEGIN
  du:=TeX_dYCoord*(y-YminGlb)/(YmaxGlb-YminGlb);
  TeX_ScalY:=TeX_CoordY+Trunc(du);
END;
{-----------------------------------------------------------------------------}

PROCEDURE TeX_Closegraph;
BEGIN
  WriteLn(TeX_File,'\end{picture}\endgroup');
  WriteLn(TeX_File,'%%%%%%%%%%%%%%%%% End of Picture %%%%%%%%%%%%%');
  Close(TeX_File);
END;
{-----------------------------------------------------------------------------}

PROCEDURE TeX_DrawNum(x:real;length_st,xpos,ypos:integer);
VAR st:workstring;
BEGIN
  x:=x*fak;Str(x:20:10,st);
  WHILE Pos(' ',st)>0 DO Delete(st,Pos(' ',st),1);
  Delete(st,length_st+1,20);
  TeX_OutTextXY(xpos,ypos,st);
END;
{-----------------------------------------------------------------------------}

PROCEDURE TeX_DrawAxis;
VAR XSet,Yset:real;
    k:integer;
BEGIN
  TeX_Rectangle(TeX_CoordX,TeX_CoordY,TeX_CoordX+TeX_dXCoord,TeX_CoordY+TeX_dYCoord);
  ScalMax;
  fak:=xfak;
  FOR k:=1 TO 9 DO
    BEGIN
      Xset:=Xmin+(Xmax-Xmin)/10*k;
      TeX_Line(TeX_ScalX(XSet),TeX_CoordY-200,TeX_ScalX(Xset),TeX_CoordY+200);
      xpos:=TeX_ScalX(XSet)-800;ypos:=TeX_CoordY;
      TeX_DrawNum(Xset,4,xpos,ypos);
    END;
  fak:=Abs(xmin);
  IF Abs(xmax)>fak THEN fak:=Abs(xmax);
  Str(XExponent:3,ch);
  ch:='E'+ch;
  TeX_OutTextXY(TeX_CoordX+TeX_dXCoord-2000,TeX_CoordY+1000,ch);
  fak:=yfak;
  FOR k:=1 TO 9 DO
    BEGIN
      Yset:=YminGlb+(YmaxGlb-YminGlb)/10*k;
      TeX_Line(TeX_CoordX-200,TeX_ScalY(Yset),TeX_CoordX+200,TeX_ScalY(Yset));
      xpos:=0;ypos:=TeX_ScalY(Yset)+200;
      TeX_DrawNum(Yset,4,xpos,ypos);
    END;
  fak:=Abs(YminGlb);
  IF Abs(YmaxGlb)>fak THEN fak:=Abs(YmaxGlb);
  Str(YExponent:3,ch);
  ch:='E'+ch;
  TeX_OutTextXY(TeX_CoordX+1000,TeX_dYCoord+TeX_CoordY-200,ch);
  FOR k:=1 TO m DO
    IF (Xmin<=tau[k]) AND (tau[k]<=Xmax) THEN
      TeX_Circle(TeX_ScalX(tau[k]),TeX_ScalY(y[k]),200);
  FOR k:=1 TO n+order DO
    IF (Xmin<=t[k]) AND (t[k]<=Xmax) THEN
      TeX_Rectangle(TeX_ScalX(t[k])-100,TeX_CoordY+100,TeX_ScalX(t[k])+100,TeX_CoordY+600);
END;  { TeX_DrawAxis }
{-----------------------------------------------------------------------------}

PROCEDURE Grid_Plot(         m,n        : integer;      { dimension           }
                             k          : integer;      { order of B-Splines  }
                             r          : integer;
                             mu         : real;
                             How        : Boolean;
                             i1,i2      : integer;
                        No_Intervals    : integer;
                        VAR t           : knotsequence;
                        VAR tau         : dataabscissae;
                        VAR y           : datavalues;
                        VAR PlotArrayD3 : PlotArrayD3Pointer
                     );

{ generates a grid of the feasible t[i1] and t[i2], computes the residual
  at this point and stores it to the matrix PlotArrayD3. The feasible grid
  of t[i1] and t[i2] will be stored in the vectors t1 and t2.
  It is assumed that i1 <= i2 and k < i1 and i2 < n+1.                        }

FUNCTION index(i, j, No_Intervals : integer) : integer;
{ computes index of two-dimensional array element A[i,j] with
  A : Array[0..No_Intervals,0..No_Intervals] of real stored as long
  one-dimensional vector                                                      }
BEGIN
  index:=i*Succ(No_Intervals)+j;
END;

VAR ii, jj          : integer;
    h1, h2          : real;
    S               : SmoothingMatrix;
    alpha           : coefficients;
    temp_t          : knotsequence;
BEGIN
  FOR ii:=1 TO n+k DO
    temp_t[ii]:=t[ii];           { saves original knotsequence }

  h1:=(t[i1+1]-t[i1-1])/(No_Intervals+2);
  h2:=(t[i2+1]-t[i2-1])/(No_Intervals+2);

  FOR ii:=0 TO No_Intervals DO
    FOR jj:=0 TO No_Intervals DO
      PlotArrayD3^[index(ii,jj,No_Intervals)]:=0.0;

  IF i1 = i2 THEN  { one dimensional plot }
    BEGIN          { t[i1] in [ t[i1-1],t[i1+1] ] }
      FOR ii:=0 TO No_Intervals DO
        BEGIN
          t[i1]:=temp_t[i1-1] + h1 * (ii+1);

          IF how=true THEN
            l2Appr(m,n,k,t,tau,y,alpha,Res)
          ELSE
            BEGIN
              Compute_Smoothing_Matrix(t,n,k,r,S);
              Compute_Smoothing_Spline(m,n,k,t,r,mu,tau,y,S,alpha,RES);
            END;
          PlotArrayD3^[index(ii,0,No_Intervals)]:= RES;
        END;
    END
  ELSE
  IF i1+1 = i2 THEN  { two dimensional plot }
    BEGIN            { t[i1],t[i2] in [ t[i1-1],t[i2+1] ]^2, t[i1] <= t[i2] }
      FOR ii:=0 TO No_Intervals DO
        BEGIN
          t[i1]:=temp_t[i1-1]+ h1 * (ii+1);

          FOR jj:=ii TO No_Intervals DO
            BEGIN
              t[i2]:=temp_t[i2-1]+ h2 * (jj+1);

              IF how=true THEN
                l2Appr(m,n,k,t,tau,y,alpha,Res)
              ELSE
                BEGIN
                  Compute_Smoothing_Matrix(t,n,k,r,S);
                  Compute_Smoothing_Spline(m,n,k,t,r,mu,tau,y,S,alpha,RES);
                END;
              PlotArrayD3^[index(ii,jj,No_Intervals)]:=Res;
            END;
        END;
      FOR ii:=0 TO No_Intervals DO
        FOR jj:=0 TO ii-1 DO
           PlotArrayD3^[index(ii,jj,No_Intervals)]:=PlotArrayD3^[index(jj,ii,No_Intervals)];
    END
  ELSE
    BEGIN  { two dimensional plot }
           { t[i1],t[i2] in [ t[i1-1],t[i1+1] x [ t[i2-1],t[i2+1] ] }
      FOR ii:=0 TO No_Intervals DO
        BEGIN
          t[i1]:=temp_t[i1-1]+ h1 * (ii+1);

          FOR jj:=0 TO No_Intervals DO
            BEGIN
              t[i2]:=temp_t[i2-1]+ h2 * (jj+1);

              IF how=true THEN
                l2Appr(m,n,k,t,tau,y,alpha,Res)
              ELSE
                BEGIN
                  Compute_Smoothing_Matrix(t,n,k,r,S);
                  Compute_Smoothing_Spline(m,n,k,t,r,mu,tau,y,S,alpha,RES);
                END;
              PlotArrayD3^[index(ii,jj,No_Intervals)]:=Res;
            END;
        END;
    END;

  FOR ii:=1 TO n+k DO
    t[ii]:=temp_t[ii];           { restores original knotsequence }
END;

{-----------------------------------------------------------------------------}

PROCEDURE graphik;
TYPE PlotArrayD2TYPE = ARRAY [1..1] OF real;
VAR PlotArrayD2                   : ^PlotArrayD2TYPE;
    PlotArrayD3                   : PlotArrayD3Pointer;
    i1,i2, j, plot_derivative_no  : integer;
    IncX,h1,h2                    : real;

BEGIN   { Graphic }
  Hardcopy:=false;Xmin:=t[1];Xmax:=t[n+order];
  REPEAT
    TextBackground(MyBackground);ClrScr;
    MenuLength:=4;
    mm[0,0]:='Graphic';
    mm[1,0]:='2D-Plot';
    mm[1,1]:='Two-dimensional presentation of the spline and its derivatives';
    mm[2,0]:='2D-Plot with TeX Output';
    mm[2,1]:='Generate File with emTeX \special commands to draw TeX picture';
    mm[3,0]:='Hardcopy';
    mm[3,1]:='Switch Hardcopy On/Off';
    mm[4,0]:='3D-Plot';
    mm[4,1]:='Three-dimensional representation of the residual sum';
    IF Hardcopy THEN mm[3,0]:=mm[3,0]+' on' ELSE mm[3,0]:=mm[3,0]+' off';
    ProgramNo:=Menu(MenuLength,mm,MyMenuBackground,MyMenuForeground);
    CASE ProgramNo OF
      0:Exit;
      1:BEGIN

        REPEAT
          TextBackground(MyBackground);
          ClrScr;Box(20,60,2,4,MyMenuBackground,MyMenuForeground);
          Msg(35,3,'Graphic');
          Box(5,75,8,22,Black,White);
          GotoXY(30,10);Write('Enter a new interval or press <ENTER>');
          GotoXY(10,12);
          ReadReal('X minimal= ',Xmin);
          GotoXY(10,14);
          ReadReal('X maximal= ',Xmax);
          GotoXY(10,16);
          plot_derivative_no:=0;
          ReadInt('Plot r-th derivative of spline: ',plot_derivative_no);
          Flag:=((Xmin>=t[1]) AND (Xmax<=t[n+order]) AND (Xmin<Xmax));
          IF NOT Flag THEN
            BEGIN
              GotoXY(30,18);
              WriteLn(Chr(7),Chr(7),'Cannot be!');
              Wait(20000);
            END;
        UNTIL Flag;

        InitGR;
        SetBkColor(black);
        SetWorldCoord;

        { allocate memory for array of function values }
        GetMem(PlotArrayD2,dXCoord*SizeOf(real));

        { compute function values }
        IncX:=(Xmax-Xmin)/(dXCoord-1);
        IF plot_derivative_no=0 THEN
          BEGIN
            FOR k:=1 TO dXCoord DO
              BEGIN
                Xorig:=Xmin+IncX*(k-1);
                interval(Xorig,n+1,t,left,out_of_interval);
                Yorig:=Spline_Value(alpha,t,Xorig,left,order);
                PlotArrayD2^[k]:=Yorig;
              END;
          END
        ELSE
          BEGIN
            FOR k:=1 TO dXCoord DO
              BEGIN
                Xorig:=Xmin+IncX*(k-1);
                interval(Xorig,n+1,t,left,out_of_interval);
                Yorig:=Spline_Value_deriv(alpha,t,Xorig,plot_derivative_no,order);
                PlotArrayD2^[k]:=Yorig;
              END;
          END;

        { find extrema of Spline in the given interval }
        YMaxGlb:=PlotArrayD2^[1];
        YMinGlb:=PlotArrayD2^[1];
        FOR k:=2 TO dXCoord DO
          BEGIN
            Yorig:=PlotArrayD2^[k];
            IF Yorig > YMaxGlb THEN YMaxGlb:=Yorig;
            IF Yorig < YMinGlb THEN YMinGlb:=Yorig;
          END;

        { find extrema of data values in the given interval }
        left_min:=1;
        WHILE Xmin > tau[left_min] DO
          Inc(left_min);
        left_max:=m;
        WHILE Xmax <= tau[left_max] DO
          Dec(left_max);

        Ymin:=y[left_min];
        Ymax:=y[left_min];

        FOR i:=left_min TO left_max DO
          BEGIN
            IF y[i] < Ymin THEN
              Ymin:=y[i];
            IF y[i] > Ymax THEN
              Ymax:=y[i];
          END;
        { extrema of y[i] in [Xmin,Xmax] found }

        IF plot_derivative_no=0 THEN
          BEGIN
            IF Ymax > YmaxGlb THEN YmaxGlb:=Ymax;
            IF Ymin < YminGlb THEN YminGlb:=Ymin;
          END;

        YmaxGlb:=YmaxGlb+0.02*(YmaxGlb-YminGlb);
        YminGlb:=YminGlb-0.02*(YmaxGlb-YminGlb);

        SetColor(White);
        SetLineStyle(SolidLn,White,Normwidth);
        ClearViewPort;

        { plot values in PlotArrayD2 }
        Yorig:=PlotArrayD2^[1];
        MoveTo(ScalX(Xmin),ScalY(Yorig));
        FOR k:=2 TO dXCoord DO
          BEGIN
            Xorig:=Xmin+IncX*(k-1);
            Yorig:=PlotArrayD2^[k];
            LineTo(ScalX(Xorig),ScalY(Yorig));
          END;

        { deallocate memory for array of function values }
        FreeMem(PlotArrayD2,dXCoord*SizeOf(real));

        SetColor(lightgray);SetLineStyle(SolidLn,black,Normwidth);
        DrawAxis(plot_derivative_no);
        IF Hardcopy THEN
          BEGIN
            WriteLn(LST,'     Least Squares Approximation with B-Splines');
            WriteLn(LST);GetDate(Year,Month,Day,DayofWeek);
            WriteLn(LST,'              Date: ',Day,'.',Month,'.',year);
            WriteLn(LST);WriteLn(LST,'  ',HeadLine);WriteLn(LST);
            WriteLn(LST,Chr(12));
            Wait(200000);
            HardcopyON;
          END;
        OutTextXY(50,0,'Press a key to continue');
        REPEAT UNTIL KeyPressed;
        CloseGraph;
      END; {case 1 }
      2:BEGIN

        REPEAT
          TextBackground(MyBackground);
          ClrScr;Box(20,60,2,4,MyMenuBackground,MyMenuForeground);
          Msg(35,3,'TeX-Output');
          Box(5,75,8,22,Black,White);
          GotoXY(30,10);Write('Enter a new interval or press <ENTER>');
          GotoXY(10,12);
          ReadReal('X minimal= ',Xmin);
          GotoXY(10,14);
          ReadReal('X maximal= ',Xmax);
          GotoXY(10,16);
          plot_derivative_no:=0;
          ReadInt('Plot r-th derivative of spline: ',plot_derivative_no);
          Flag:=((Xmin>=t[1]) AND (Xmax<=t[n+order]) AND (Xmin<Xmax));
          IF NOT Flag THEN
            BEGIN
              GotoXY(30,18);
              WriteLn(Chr(7),Chr(7),'Cannot be!');
              Wait(20000);
            END;
        UNTIL Flag;

        Box(20,60,17,19,White,Black);
        TextColor(Red);
        Msg(23,18,'Computing TeX Graphic. Please wait!');

        TeX_Initgraph;
        TeX_SetWorldCoord;

        { allocate memory for array of function values }
        GetMem(PlotArrayD2,(TeX_dXCoord DIV 50)*SizeOf(real));

        { compute function values }
        IncX:=(Xmax-Xmin)/((TeX_dXCoord DIV 50) -1);
        IF plot_derivative_no=0 THEN
          BEGIN
            FOR k:=1 TO (TeX_dXCoord DIV 50) DO
              BEGIN
                Xorig:=Xmin+IncX*(k-1);
                interval(Xorig,n+1,t,left,out_of_interval);
                Yorig:=Spline_Value(alpha,t,Xorig,left,order);
                PlotArrayD2^[k]:=Yorig;
              END;
          END
        ELSE
          BEGIN
            FOR k:=1 TO (TeX_dXCoord DIV 50) DO
              BEGIN
                Xorig:=Xmin+IncX*(k-1);
                interval(Xorig,n+1,t,left,out_of_interval);
                Yorig:=Spline_Value_deriv(alpha,t,Xorig,plot_derivative_no,order);
                PlotArrayD2^[k]:=Yorig;
              END;
          END;

        { find extrema of Spline in the given interval }
        YMaxGlb:=PlotArrayD2^[1];
        YMinGlb:=PlotArrayD2^[1];
        FOR k:=2 TO (TeX_dXCoord DIV 50) DO
          BEGIN
            Yorig:=PlotArrayD2^[k];
            IF Yorig > YMaxGlb THEN YMaxGlb:=Yorig;
            IF Yorig < YMinGlb THEN YMinGlb:=Yorig;
          END;

        { find extrema of data values in the given interval }
        left_min:=1;
        WHILE Xmin > tau[left_min] DO
          Inc(left_min);
        left_max:=m;
        WHILE Xmax <= tau[left_max] DO
          Dec(left_max);

        Ymin:=y[left_min];
        Ymax:=y[left_min];

        FOR i:=left_min TO left_max DO
          BEGIN
            IF y[i] < Ymin THEN
              Ymin:=y[i];
            IF y[i] > Ymax THEN
              Ymax:=y[i];
          END;
        { extrema of y[i] in [Xmin,Xmax] found }

        IF plot_derivative_no=0 THEN
          BEGIN
            IF Ymax > YmaxGlb THEN YmaxGlb:=Ymax;
            IF Ymin < YminGlb THEN YminGlb:=Ymin;
          END;

        IF YmaxGlb >0 THEN YmaxGlb:=1.02*YmaxGlb
        ELSE YmaxGlb:=0.98*YmaxGlb;
        IF YminGlb <0 THEN YminGlb:=1.02*YminGlb
        ELSE YminGlb:=0.98*YminGlb;

        Yorig:=PlotArrayD2^[1];
        TeX_MoveTo(TeX_ScalX(Xmin),TeX_ScalY(Yorig));
        FOR k:=2 TO (TeX_dXCoord DIV 50) DO
          BEGIN
            Xorig:=Xmin+IncX*(k-1);
            Yorig:=PlotArrayD2^[k];
            TeX_LineTo(TeX_ScalX(Xorig),TeX_ScalY(Yorig));
          END;

        { deallocate memory for array of function values }
        FreeMem(PlotArrayD2,(TeX_dXCoord DIV 50) *SizeOf(real));

        TeX_DrawAxis;
        TeX_closegraph;
      END; {case 2 }
      3:BEGIN
        HardCopy:=NOT Hardcopy;
        IF Hardcopy THEN
          BEGIN
            TextBackground(MyBackground);
            ClrScr;
            Box(20,60,2,4,MyMenuBackground,MyMenuForeground);
            Msg(35,3,'Hardcopy');
            Box(5,75,8,22,Black,White);
            Msg(7,12,'Enter Headline ! ');
            GotoXY(7,15);
            ReadLn(Headline);
            Msg(6,18,'Press <ENTER> , when printer is ready !');
            REPEAT UNTIL KeyPressed;
          END;
      END; { case 2 }
      4:BEGIN
        TextBackground(MyBackground);
        ClrScr;Box(20,60,2,4,MyMenuBackground,MyMenuForeground);
        Msg(35,3,'3D-Graphic');
        Box(5,75,8,22,Black,White);
        GotoXY(20,12);
        character:='N';
        ReadChar('Smoothing Spline (y/n): ',character);
        IF UpCase(character) IN ['Y','J'] THEN how:=false ELSE how:=true;
        GotoXY(20,14);
        Write('Enter index of first inner knot:  ');
        ReadLn(i1);
        GotoXY(20,16);
        Write('Enter index of second inner knot:  ');
        ReadLn(i2);
        GotoXY(20,18);
        IF MaxAvail > 65500 THEN
          No_Intervals:=Trunc(Sqrt(65500/SizeOf(real)))-1
        ELSE
          No_Intervals:=Trunc(Sqrt(MaxAvail/SizeOf(real)))-1;
        ReadInt('Number of Intervals <= : ',No_Intervals);
        GotoXY(20,20);
        FileOutput:='N';
        ReadChar('Output to File (y/n) ? ',FileOutput);
        { allocate memory for array of function values }
        GetMem(PlotArrayD3,Succ(No_Intervals)*Succ(No_Intervals)*SizeOf(real));

        Grid_Plot(m,n,order,r,mu,how,i1,i2,No_Intervals,t,tau,y,PlotArrayD3);

        h1:=(t[i1+1]-t[i1-1])/(No_Intervals+2);
        h2:=(t[i2+1]-t[i2-1])/(No_Intervals+2);

        IF UpCase(FileOutput) IN ['Y','J'] THEN
          BEGIN
            Assign(TeX_File, 'free.dat');
            {$I-}
            Rewrite(TeX_File);
            {$I+}
            IF IOResult<>0 THEN
              BEGIN
                WriteLn('Error in Graphic');
                Exit;
              END;
            FOR i:=0 TO No_Intervals DO
              BEGIN
                FOR j:=0 TO No_Intervals DO
                  WriteLn(TeX_File,t[i1-1]+ h1 * (i+1),'  ',t[i2-1]+ h2 * (j+1),'  ',PlotArrayD3^[i*Succ(No_Intervals)+j]);
                WriteLn(TeX_File);
              END;

            Close(Tex_File);
          END;

        MyPlotArray.AnzPlotX:=No_Intervals;
        MyPlotArray.AnzPlotY:=No_Intervals;
        MyPlotArray.FktAdr:=PlotArrayD3;

        D3MINIMAX(MyPlotArray);
        InitGR;
        SetBkColor(black);

        D3FUNKTIONSPLOT2(MyPlotArray);
        OutTextXY(50,0,'Press a key to continue');
        REPEAT UNTIL KeyPressed;
        D3Hoehe(MyPlotArray);



        OutTextXY(50,0,'Press a key to continue');
        REPEAT UNTIL KeyPressed;
        CloseGraph;

        { deallocate memory for array of function values }
        FreeMem(PlotArrayD3,Succ(No_Intervals)*Succ(No_Intervals)*SizeOf(real));
      END; {case 4 }

    END; {case}
  UNTIL NOT KeyPressed;
END;    { Graphic }

END.