UNIT InOut;

INTERFACE
  USES def,       { definitions of constants, types and global variables      }
       error,     { error handler                                             }
       various,   { various procedures for menu handling                      }
       math,      { low level mathematical subroutines                        }
       bspline,   { procedures to compute B-Splines and l2-approximations     }
       smooth,    { procedures to compute smoothing B-Splines                 }
       blas,      { Basic Linear Algebra Subroutines                          }
       CSSpline,  { procedures for constrained smoothing B-Splines            }
       crt,dos,Printer; { Standard Turbo-Pascal Units                         }

{-----------------------------------------------------------------------------}

PROCEDURE BSRead;
  { get data }

{-----------------------------------------------------------------------------}

PROCEDURE BSSave;
  { put data }

{-----------------------------------------------------------------------------}

PROCEDURE Correct;
  { correct data }

{-----------------------------------------------------------------------------}

PROCEDURE Parameter;
  { Change number of data values and dimension of spline space }

{-----------------------------------------------------------------------------}

IMPLEMENTATION

{-----------------------------------------------------------------------------}

TYPE Data=RECORD
       abscissa  : real;
       ordinate  : real;
     END;
     Spline=RECORD
       breakpoint  : real;
       coefficient : real;
     END;

VAR DataFile   : FILE OF Data;
    DataRec    : Data;
    SplineFile : FILE OF Spline;
    SplineRec  : Spline;
    TxtFile    : Text;
    Filename   : STRING;
    ProgramNo  : integer;
    F          : datavalues;

{-----------------------------------------------------------------------------}

PROCEDURE Directory;
VAR Dirinfo : SearchRec;
    i       : integer;

BEGIN
  FindFirst('*.*',Archive,DirInfo);
  i:=1;
  WHILE Doserror=0 DO
    BEGIN
      IF i>4 THEN
        BEGIN
          i:=1;{ WriteLn}
        END;
      Write(DirInfo.Name:20);
      FindNext(DirInfo);
      Inc(i);
    END;
  WriteLn;
END; { Directory }

{-----------------------------------------------------------------------------}

PROCEDURE GetName;
VAR info : SearchRec;
    OK   : Boolean;
BEGIN
  REPEAT
    TextBackground(MyBackground);
    ClrScr;
    TextColor(Black);
    Directory;
    Box(10,70,20,24,MyMenuBackground,MyMenuForeground);
    Msg(15,22,'Enter Filename: ');
    ReadLn(FileName);
    FindFirst(FileName,(Archive+Readonly),info);
    OK:=(DosError=0);
    IF NOT OK THEN
      BEGIN
        Write(Chr(7),Chr(7));
        msg(15,23,'Cannot find such a file');
        wait(5000);
      END;
  UNTIL OK;
END; { GetName }

{-----------------------------------------------------------------------------}

PROCEDURE Data_File_Read;
VAR i  : integer;
    ch : char;

BEGIN
  GetName;
  Box(10,70,20,24,MyMenuBackground,MyMenuForeground);
  GotoXY(15,22);
  ch:='b';
  ReadChar('Read ascii or binary file (a/b)?: ',ch);

  IF UpCase(ch) <> 'A' THEN
    BEGIN
      Assign(DataFile,FileName);
      {$I-} Reset(DataFile); {$I+}
      IF (IOResult<>0) THEN
        BEGIN
          Msg(15,23,'Cannot open file');Write(FileName);
          Wait(2000);Exit
        END;
      FOR i:=1 TO m DO
        BEGIN
          Read(DataFile,DataRec);
          WITH DataRec DO
            BEGIN
              tau[i]:= abscissa;
              y[i]  := ordinate;
            END;
        END;
      Close(DataFile);
    END
  ELSE
    BEGIN
      Assign(TxtFile,FileName);
      {$I-} Reset(TxtFile); {$I+}
      IF (IOResult<>0) THEN
        BEGIN
          Msg(15,23,'Cannot open file');Write(FileName);
          Wait(2000);Exit
        END;
      FOR i:=1 TO m DO
        BEGIN
          Read(TxtFile,tau[i]);
          Read(TxtFile,y[i]);
        END;
      Close(TxtFile);
    END;
  Msg(15,23,'Last Record loaded!');
  Write(Chr(7),Chr(7));
  Wait(2000);
END; { Data_File_Read }

{-----------------------------------------------------------------------------}

PROCEDURE titandata( VAR tau: dataabscissae;
                     VAR y:   datavalues;
                     VAR m:   integer
                     );

{ Set Titanium heat data.
  Output: First m values;   ( m = 31 Andersson )
  See de Boor 78, p. 222.

  These data represent a property of titanium as a function of temperature.}

CONST ytitan: ARRAY[1..49] OF real = (
    0.644,  0.622,  0.638,  0.649,  0.652, 0.639, 0.646, 0.657,  0.652,  0.655
  , 0.644,  0.663,  0.663,  0.668,  0.676, 0.676, 0.686, 0.679,  0.678,  0.683
  , 0.694,  0.699,  0.710,  0.730,  0.763, 0.812, 0.907, 1.044,  1.336,  1.881
  , 2.169,  2.075,  1.598,  1.211,  0.916, 0.746, 0.672, 0.627,  0.615,  0.607
  , 0.606,  0.609,  0.603,  0.601,  0.603, 0.601, 0.611, 0.601,  0.608 );

VAR i: integer;

BEGIN
  IF m > 49 THEN m := 49;
  IF m > mmax THEN
    BEGIN
      Error_Handler('Set Titan data: MMAX to small to set Titan Data !');
    END{ if m};
  FOR i := 1 TO m DO
    BEGIN
      tau[i] := 585.0 + 10.0 * i;
      y[i]   := ytitan[i];
    END; { for i}

  { tau[1] = 595, tau[49] = 1075  }

END; { titandata}

{-----------------------------------------------------------------------------}

{ Fritsch-Carlson RPN 14 radiochemical data    }
PROCEDURE RPN14Data(
                           VAR tau: dataabscissae;
                           VAR y:   datavalues;
                           VAR m:   integer);

TYPE feld9 = ARRAY[1..9] OF real;

CONST x: feld9 = ( 7.99,     8.09,       8.19,       8.7,       9.2,
                  10,       12,         15,         20 );

      f: feld9 = ( 0.0,      2.76429e-5, 4.37498e-2, 0.169183,  0.469428,
                   0.943740, 0.998636,   0.999919,   0.999994);


VAR i:integer;

BEGIN
  IF m>9 THEN
    m := 9;
  IF m>MMAX THEN
    BEGIN
      Error_Handler('Set RPN14 data: MMAX to small to set RPN14 data !');
    END;
  FOR i := 1 TO m DO
    BEGIN
      tau[i] := x[i];
      y[i] := f[i];
    END;
END; { rpn14data }

{-----------------------------------------------------------------------------}

PROCEDURE Spline_File_Read;
VAR j  : integer;
    ch : char;
    h  : real;

BEGIN
  GetName;
  Box(10,70,20,24,MyMenuBackground,MyMenuForeground);
  GotoXY(15,22);
  ch:='b';
  ReadChar('Read ascii or binary file (a/b)?: ',ch);

  IF UpCase(ch) <> 'A' THEN
    BEGIN
      Assign(SplineFile,FileName);
      {$I-} Reset(SplineFile); {$I+}
      IF (IOResult<>0) THEN
        BEGIN
          Msg(15,23,'Cannot open file');Write(FileName);
          Wait(2000);Exit
        END;
      FOR j:=1 TO n+order DO
        BEGIN
          Read(SplineFile,SplineRec);
          WITH SplineRec DO
            BEGIN
              t[j]:=breakpoint;
              IF j<=n THEN
                alpha[j]:=coefficient;
            END;
        END;
      Close(SplineFile);
    END
  ELSE
    BEGIN
      Assign(TxtFile,FileName);
      {$I-} Reset(TxtFile); {$I+}
      IF (IOResult<>0) THEN
        BEGIN
          Msg(15,23,'Cannot open file');Write(FileName);
          Wait(2000);Exit
        END;
      FOR j:=1 TO n+order DO
        BEGIN
          Read(TxtFile,t[j]);
          Read(TxtFile,h);
          IF j<=n THEN
            alpha[j]:=h;
        END;
      Close(TxtFile);
    END;
  Msg(15,23,'Last Record loaded!');
  Write(Chr(7),Chr(7));
  Wait(2000);
END; { Spline_File_Read }

{-----------------------------------------------------------------------------}

PROCEDURE Spline_Con_Read;
VAR j:integer;

BEGIN
  TextBackground(MyBackground);
  ClrScr;Box(2,79,2,24,MyMenuBackground,MyMenuForeground);
  Box(10,70,2,4,Black,White);
  Msg(25,3,'Reading Spline Parameter');
  FOR j:=1 TO n+order DO
    BEGIN
      Box(10,75,10,16,Red,White);
      Msg(20,11,'Element No.: ');Write(j:3);
      IF j>n THEN
        BEGIN
          Msg(20,13,'breakpoint       : ');ReadLn(t[j]);
        END
      ELSE
        BEGIN
          Msg(20,13,'breakpoint       : ');ReadLn(t[j]);
          Msg(20,14,'coefficient      : ');ReadLn(alpha[j]);
        END;
    END;
END; { Spline_Con_Read }

{-----------------------------------------------------------------------------}

PROCEDURE Data_Con_Read;
VAR i:integer;

BEGIN
  TextBackground(MyBackground);
  ClrScr;Box(2,79,2,24,MyMenuBackground,MyMenuForeground);
  Box(10,70,2,4,Black,White);
  Msg(25,3,'Reading data values from console');
  FOR i:=1 TO m DO
    BEGIN
      Box(10,75,10,16,Red,White);
      Msg(20,11,'Element No.: ');Write(i:3);
      Msg(20,13,'abscissa : ');ReadLn(tau[i]);
      Msg(20,14,'ordinate : ');ReadLn(y[i]);
    END;
END; { Data_Con_Read }

{-----------------------------------------------------------------------------}

PROCEDURE Transform_Bounds(
                      n, k, p       : integer;
                  VAR t             : knotsequence;
                  VAR Active_Bounds : BLSConstr;
                  VAR lower, upper  : BLSbounds;
                  VAR Bounds        : BLSConstr;
                  VAR l,u           : BLSbounds
                        );

{ Transform bounds on derivatives in bounds on coefficients

Input:  bounds on derivatives
        l[i] <= s^(p)(x) <= u[i]              for x in [t[i],t[i+1])
                                              i=k,...,n

        if Bounds[i].lower_bound=FALSE     l[i] = -infinity
        if Bounds[i].upper_bound=FALSE     u[i] = +infinity

Output: bounds on coefficients alpha^(p)
        lower[j] <= alpha^(p)[j] <= upper[j]  j=p+1,...,n

        if Active_Bounds[j].lower_bound=FALSE  lower[j] = -infinity
        if Active_Bounds[j].upper_bound=FALSE  upper[j] = +infinity

        We store lower[p+1],...,lower[n] in lower[1],...,lower[n-p]
                 Active_bounds[p+1],...,Active_bounds[n] in
                 Active_bounds[1],...,Active_bounds[n-p]                      }

VAR i,j,imin,imax,ilower_bound_index,iupper_bound_index : integer;
    ilower_bound,iupper_bound                           : boolean;
    max_lower,min_upper                                 : real;

BEGIN
  FOR j:=1 TO n DO
    BEGIN
      lower[j]:=0.0;
      upper[j]:=0.0;
      WITH active_Bounds[j] DO
        BEGIN
          bound:=false;
          lower_bound:=false;
          upper_bound:=false;
        END;
    END; { Initialization }

  FOR j:=p+1 TO n DO
    BEGIN
      imin:=max(j,k);
      imax:=min(j+k-p-1,n); { Index set W_j=(imin,...,imax) }

      ilower_bound:=false;
      iupper_bound:=false;
      { ilower_bound=false  if max( l[i] : i in (imin,...,imax) ) = -infinity
        iupper_bound=false  if min( u[i] : i in (imin,...,imax) ) = +infinity }

      FOR i:=imin TO imax DO
        BEGIN
          IF bounds[i].lower_bound=true THEN
            BEGIN
              ilower_bound:=true;
              ilower_bound_index:=i;
            END;
          IF bounds[i].upper_bound=true THEN
            BEGIN
              iupper_bound:=true;
              iupper_bound_index:=i;
            END;
        END;

      IF ilower_bound=true THEN
        BEGIN
          { at least one index (ilower_bound_index) is greater than -infinity }
          max_lower:=l[ilower_bound_index];
          FOR i:=ilower_bound_index-1 DOWNTO imin DO
            BEGIN
              IF bounds[i].lower_bound=true THEN { We can compare the values }
                BEGIN
                  IF max_lower<l[i] THEN
                    max_lower:=l[i];
                END;
            END;
          { Now we have found max( l[i] : i in (imin,...,imax)) > -infinity }
          lower[j-p]:=max_lower;
          Active_bounds[j-p].lower_bound:=true;
        END;

      IF iupper_bound=true THEN
        BEGIN
          { at least one index (iupper_bound_index) is lower than +infinity }
          min_upper:=u[iupper_bound_index];
          FOR i:=iupper_bound_index-1 DOWNTO imin DO
            BEGIN
              IF bounds[i].upper_bound=true THEN { We can compare the values }
                BEGIN
                  IF min_upper>u[i] THEN
                    min_upper:=u[i];
                END;
            END;
          { Now we have found min( u[i] : i in (imin,...,imax)) < +infinity }
          upper[j-p]:=min_upper;
          Active_bounds[j-p].upper_bound:=true;
        END;
    END; { for j }
END; { Transform_Bounds }

{-----------------------------------------------------------------------------}

PROCEDURE Input_Bounds(
                      n, k, p       : integer;
                  VAR t             : knotsequence;
                  VAR Active_Bounds : BLSConstr;
                  VAR lower, upper  : BLSbounds
                      );

{ Input bounds on derivatives, transform in bounds on coefficients

Output: bounds on coefficients alpha^(p)
        lower[j] <= alpha^(p)[j] <= upper[j]  j=p+1,...,n

        if Active_Bounds[j].lower_bound=FALSE  lower[j] = -infinity
        if Active_Bounds[j].upper_bound=FALSE  upper[j] = +infinity

        We store lower[p+1],...,lower[n] in lower[1],...,lower[n-p]
                 Active_bounds[p+1],...,Active_bounds[n] in
                 Active_bounds[1],...,Active_bounds[n-p]                      }

VAR l,u                   : BLSBounds; { bounds on derivatives }
    bounds                : BLSConstr;
    i,left_min,left_max   : integer;
    loop                  : Boolean;
    ch                    : char;
    y_lower,y_upper,interval_beg,interval_end : real;

BEGIN
  FOR i:=1 TO n DO
    BEGIN
      l[i]:=0.0;
      u[i]:=0.0;
      WITH bounds[i] DO
        BEGIN
          bound:=false;
          lower_bound:=false;
          upper_bound:=false;
        END;
      END; { Initialization }

  y_lower := 0.0;
  interval_beg := t[k];
  interval_end := t[n+1];
  { assume constraint is imposed on whole interval }

  TextBackground(MyBackground);
  ClrScr;
  Box(20,60,2,4,MyMenuBackground,MyMenuForeground);
  Msg(25,3,'Set bounds on derivatives');
  Box(5,75,8,22,Black,White);

  GotoXY(10,10);
  ch:='Y';
  ReadChar('Impose lower bounds (y/n) : ',ch);
  IF UpCase(ch) IN ['Y','J'] THEN
    loop:=true
  ELSE
    loop:=false;

  WHILE loop DO
    BEGIN
      TextBackground(MyBackground);
      ClrScr;
      Box(20,60,2,4,MyMenuBackground,MyMenuForeground);
      Msg(25,3,'Set lower bounds on derivatives');
      Box(5,75,8,22,Black,White);

      GotoXY(10,10);
      ReadReal('From (left interval bound) : ',interval_beg);
      interval(interval_beg,n+1,t,left_min,out_of_interval);
      GotoXY(10,11);
      ReadInt('From left knot index : ',left_min);

      GotoXY(10,13);
      ReadReal('To (right interval bound)  : ',interval_end);
      interval(interval_end,n+1,t,left_max,out_of_interval);
      IF interval_end>=t[n+1] THEN left_max:=n+1;
      GotoXY(10,14);
      ReadInt('To right knot index : ',left_max);

      GotoXY(10,16);
      ch:='Y';
      ReadChar('Impose lower bound (y/n) : ',ch);
      IF UpCase(ch) IN ['Y','J'] THEN
        BEGIN
          GotoXY(10,17);
          ReadReal('Value of lower bound : ',y_lower);
          FOR i:=left_min TO left_max-1 DO
            BEGIN
              l[i]:=y_lower;
              bounds[i].lower_bound:=TRUE;
            END;
        END;

      IF left_max <= n THEN
        BEGIN
          GotoXY(10,21);
          ch:='Y';
          ReadChar('Another Interval with lower bounds ? (y/n): ',ch);
          Loop := ( UpCase(ch) IN ['Y','J'] );

          interval_beg:= t[left_max];
          interval_end:= t[n+1];
        END
      ELSE
        loop:=false;
    END; {loop}

  y_upper := 0.0;
  interval_beg := t[k];
  interval_end := t[n+1];
  { assume constraint is imposed on whole interval }

  TextBackground(MyBackground);
  ClrScr;
  Box(20,60,2,4,MyMenuBackground,MyMenuForeground);
  Msg(25,3,'Set bounds on derivatives');
  Box(5,75,8,22,Black,White);

  GotoXY(10,10);
  ch:='Y';
  ReadChar('Impose upper bounds (y/n) : ',ch);
  IF UpCase(ch) IN ['Y','J'] THEN
    loop:=true
  ELSE
    loop:=false;

  WHILE loop DO
    BEGIN
      TextBackground(MyBackground);
      ClrScr;
      Box(20,60,2,4,MyMenuBackground,MyMenuForeground);
      Msg(25,3,'Set upper bounds on derivatives');
      Box(5,75,8,22,Black,White);

      GotoXY(10,10);
      ReadReal('From (left interval bound) : ',interval_beg);
      interval(interval_beg,n+1,t,left_min,out_of_interval);
      GotoXY(10,11);
      ReadInt('From left knot index : ',left_min);

      GotoXY(10,13);
      ReadReal('To (right interval bound)  : ',interval_end);
      interval(interval_end,n+1,t,left_max,out_of_interval);
      IF interval_end>=t[n+1] THEN left_max:=n+1;
      GotoXY(10,14);
      ReadInt('To right knot index : ',left_max);

      GotoXY(10,16);
      ch:='Y';
      ReadChar('Impose upper bound (y/n) : ',ch);
      IF UpCase(ch) IN ['Y','J'] THEN
        BEGIN
          GotoXY(10,17);
          ReadReal('Value of upper bound : ',y_upper);
          FOR i:=left_min TO left_max-1 DO
            BEGIN
              u[i]:=y_upper;
              bounds[i].upper_bound:=TRUE;
            END;
        END;

      IF left_max <= n THEN
        BEGIN
          GotoXY(10,21);
          ch:='Y';
          ReadChar('Another Interval with upper bounds ? (y/n): ',ch);
          Loop := ( UpCase(ch) IN ['Y','J'] );

          interval_beg:= t[left_max];
          interval_end:= t[n+1];
        END
      ELSE
        loop:=false;
    END; {loop}

  { Now we have all necessary information to transform
    the bounds on derivatives into bounds on coefficients of spline }
  Transform_Bounds(n,k,p,t,Active_Bounds,lower, upper,bounds,l,u);
END;

{-----------------------------------------------------------------------------}

PROCEDURE Check_Bounds(
                      n, k, p       : integer;
                  VAR Active_Bounds : BLSConstr;
                  VAR lower, upper  : BLSbounds
                      );

{ Check consistency relation L <= U and strong consistency relation L < U     }

VAR j : integer;

BEGIN
  FOR j:=p+1 TO n DO
    BEGIN
      WITH Active_Bounds[j-p] DO
        IF lower_bound AND upper_bound THEN
          BEGIN
            IF NOT(lower[j-p] <= upper[j-p]) THEN { no consistency }
              IF NOT(lower[j-p] < upper[j-p]) THEN
                BEGIN { no strong consistency}
                  TextBackground(MyBackground);
                  ClrScr;
                  Box(20,60,2,4,MyMenuBackground,MyMenuForeground);
                  Msg(25,3,'Check consistency of constraints');
                  Box(5,75,8,22,Black,White);
                  Msg(10,11,'Strong consistency relation L < U does not hold!');
                  GotoXY(10,12);
                  Write('Index of constraint: ',j);
                  Wait(5000);
                END
              ELSE
                BEGIN
                  TextBackground(MyBackground);
                  ClrScr;
                  Box(20,60,2,4,MyMenuBackground,MyMenuForeground);
                  Msg(25,3,'Check consistency of constraints');
                  Box(5,75,8,22,Black,White);
                  Msg(10,11,'Consistency relation L <= U does not hold!');
                  GotoXY(10,12);
                  Write('Index of constraint: ',j);
                  Wait(5000);
                END;
          END;
    END; { for j }
END;

{-----------------------------------------------------------------------------}

PROCEDURE Set_Bounds(
                      n, k        : integer;
                  VAR t           : knotsequence;
                  VAR Active_Bounds: BLSConstr;
                  VAR lower, upper: BLSbounds;
                  VAR TypeOfConstraints: char
                                 );

{ This procedure sets bounds on coefficents for constrained spline
  approximation }

VAR i,p : integer;

BEGIN
  REPEAT
    TextBackground(MyBackground);
    ClrScr;MenuLength:=8;
    mm[0,0]:='Set bounds for derivatives';
    mm[1,0]:='Positivity';
    mm[2,0]:='Monotonicity';
    mm[3,0]:='Convexity';
    mm[4,0]:='Interval-wise bounds for s';
    mm[5,0]:='Interval-wise bounds for s''';
    mm[6,0]:='Interval-wise bounds for s''''';
    mm[7,0]:='Check consistency of bounds';
    mm[8,0]:='Clear bounds for derivatives';
    mm[1,1]:='s(x) >= 0 for all x in [a,b]';
    mm[2,1]:='s''(x) >= 0 for all x in [a,b]';
    mm[3,1]:='s''''(x) >= 0 for all x in [a,b]';
    mm[4,1]:='Impose bounds l <= s(x) <= u for subintervals of [a,b]';
    mm[5,1]:='Impose bounds l <= s''(x) <= u for subintervals of [a,b]';
    mm[6,1]:='Impose bounds l <= s''''(x) <= u for subintervals of [a,b]';
    mm[7,1]:='Check (strong) consistency L <= U (L < U) of bounds';
    mm[8,1]:='Clear all constraints on derivatives';
    ProgramNo:=Menu(MenuLength,mm,MyMenuBackground,MyMenuForeground);
    CASE ProgramNo OF
      0:Exit;
      1:BEGIN
          TypeOfConstraints:='P';
          FOR i:=1 TO n DO
            BEGIN
              WITH active_bounds[i] DO
                BEGIN
                  lower_bound:=true;
                  bound:=true;
                END;
              lower[i]:=0.0;
            END;
        END;
      2:BEGIN
          TypeOfConstraints:='M';
          FOR i:=1 TO n-1 DO
            BEGIN
              WITH active_bounds[i] DO
                BEGIN
                  lower_bound:=true;
                  bound:=true;
                END;
              lower[i]:=0.0;
            END;
        END;
      3:BEGIN
          TypeOfConstraints:='C';
          FOR i:=1 TO n-2 DO
            BEGIN
              WITH active_bounds[i] DO
                BEGIN
                  lower_bound:=true;
                  bound:=true;
                END;
              lower[i]:=0.0;
            END;
        END;
      4:BEGIN
          TypeOfConstraints:='0';
          Input_Bounds(n,k,0,t,Active_Bounds,lower,upper);
        END;
      5:BEGIN
          TypeOfConstraints:='1';
          Input_Bounds(n,k,1,t,Active_Bounds,lower,upper);
        END;
      6:BEGIN
          TypeOfConstraints:='2';
          Input_Bounds(n,k,2,t,Active_Bounds,lower,upper);
        END;
      7:BEGIN
          CASE TypeOfConstraints OF
            'P':p:=0;
            'M':p:=1;
            'C':p:=2;
          END;
          IF TypeOfConstraints IN ['0','1','2'] THEN
            p:=integer(TypeOfConstraints)-integer('0');
          Check_Bounds(n,k,p,Active_Bounds,lower,upper);
        END;
      8:BEGIN
          TypeOfConstraints:='N';
          FOR i:=1 TO n DO
            BEGIN
              WITH active_bounds[i] DO
                BEGIN
                  lower_bound:=false;
                  upper_bound:=false;
                  bound:=false;;
                END;
              lower[i]:=0.0;
              upper[i]:=0.0;
            END;
        END;
    END; {case}
  UNTIL KeyPressed;
END;

{-----------------------------------------------------------------------------}

PROCEDURE BSRead;
VAR i,j,pj,nold : integer;
    ch          : char;
    d           : datavalues;
    maxy        : real;
BEGIN
  REPEAT
    TextBackground(MyBackground);
    ClrScr;MenuLength:=4;
    mm[0,0]:='Input';
    mm[1,0]:='Read data values';
    mm[2,0]:='Read spline parameter';
    mm[3,0]:='Free knots';
    mm[4,0]:='Set constraints';
    mm[1,1]:='tau[i], y[i]      i=1,..,m ';
    mm[2,1]:='t[j],  j=1,..,n+order;  alpha[j],  j=1,..,n';
    mm[3,1]:='Set indizes of free knots and minimum distance between knots';
    mm[4,1]:='Set number and intervals with constraints on derivatives';
    ProgramNo:=Menu(MenuLength,mm,MyMenuBackground,MyMenuForeground);
    CASE ProgramNo OF
      0:Exit;
      1:BEGIN
        TextBackground(MyBackground);
        ClrScr;MenuLength:=3;
        mm[0,0]:='Read Data Values';
        mm[1,0]:='Disk';
        mm[1,1]:='Read ascii or binary file from disk';
        mm[2,0]:='Keyboard';
        mm[2,1]:='Input data from keyboard';
        mm[3,0]:='Testfunction';
        mm[3,1]:='Generate data using a testfunction';
        ProgramNo:=Menu(Menulength,mm,MyMenuBackground,MyMenuForeground);
        CASE ProgramNo OF
          0:Exit;
          1:Data_File_Read;
          2:Data_Con_Read;
          3:BEGIN
            TextBackground(MyBackground);
            ClrScr;MenuLength:=2;
            mm[0,0]:='Testfunction';
            mm[1,0]:='Titan Data';
            mm[2,0]:='RPN 14 Data';
            mm[1,1]:='Set Titanium Heat Data, see de Boor';
            mm[2,1]:='Set Fritsch-Carlson RPN 14 chemical data';
            ProgramNo:=Menu(MenuLength,mm,MyMenuBackground,MyMenuForeground);
            CASE ProgramNo OF
              0:Exit;
              1:Titandata(tau,y,m);
              2:RPN14data(tau,y,m);
            END;
          END;
        END;
      END;
      2:BEGIN
        TextBackground(MyBackground);MenuLength:=3;ClrScr;
        mm[0,0]:='Read Spline Parameter';
        mm[1,0]:='Disk';
        mm[1,1]:='Read ascii or binary file from disk';
        mm[2,0]:='Keyboard';
        mm[2,1]:='Input parameter from keyboard';
        mm[3,0]:='Test knotsequence';
        mm[3,1]:='Set one of several standard knotsequences';
        ProgramNo:=Menu(MenuLength,mm,MyMenuBackground,MyMenuForeground);
        CASE ProgramNo OF
          0:Exit;
          1:Spline_File_Read;
          2:Spline_Con_Read;
          3:BEGIN
            TextBackground(MyBackground);MenuLength:=3;ClrScr;
            mm[0,0]:='Generate Test Knot Sequence';
            mm[1,0]:='equidistant wrt. abscissae';
            mm[1,1]:='equidistant with respect to abscissae';
            mm[2,0]:='equidistant wrt. chord length';
            mm[2,1]:='equidistant with respect to chord length, see Cox,Harris,Jones 87';
            mm[3,0]:='per data point';
            mm[3,1]:='knots at data points';
            ProgramNo:=Menu(MenuLength,mm,MyMenuBackground,MyMenuForeground);
            CASE ProgramNo OF
              1:BEGIN
                TextBackground(MyBackground);
                ClrScr;
                TextColor(black);
                Msg(10,5, '  t[j]:= tau[1]                                ');
                Msg(10,7, '                                 j =1.. order  ');
                Msg(10,9, '  t[j]:= tau[1] + (j-order+1) * (tau[m]-tau[1])/(n-order+1)');
                Msg(10,11,'                                 j=order+1..n  ');
                Msg(10,13,'  t[j]:= tau[m]                                ');
                Msg(10,15,'                                j=n+1..n+order');
                Msg(3,18, ' Knotsequence doesn`t necessary fulfill the Whitney-Schoenberg condition');
                FOR j:=1 TO order DO
                  t[j]:=tau[1];
                FOR j:=order+1 TO n DO
                  t[j]:=tau[1]+ (j-order) * (tau[m]-tau[1]) / (n-order +1);
                FOR j:=n+1 TO n+order DO
                  t[j]:=tau[m];
                FOR j:=1 TO n DO
                  alpha[j]:=0.0;
                Wait(10000);
              END;
              2:BEGIN
                TextBackground(MyBackground);
                ClrScr;
                TextColor(black);
                Msg(3,18, ' Knotsequence doesn`t necessary fulfill the Whitney-Schoenberg condition');
                { We must scale the variables tau and y too. They must have
                  the same dimension to get an effect in the chord length }
                maxy:=Abs(y[1]);
                FOR i:=2 TO m DO
                  IF Abs(y[i])>maxy THEN maxy:=Abs(y[i]);

                d[1]:=0.0;
                FOR i:=2 TO m DO
                  d[i]:=d[i-1]+Sqrt(Sqr((tau[i]-tau[i-1])/tau[m])+Sqr((y[i]-y[i-1])/maxy));
                { d[i] contains distance from i-th data point to first
                  data point via straight line segments                }
                FOR i:=1 TO m DO
                  d[i]:=d[i]/d[m];
                { scaling the variables to [0,1] }

                { We now locate the n-order inner knots t[n+order],...,t[n]
                  equidistant with respect to the distance d[i] }
                FOR j:=1 TO n-order DO
                  BEGIN
                    pj:=1;
                    WHILE d[pj]< (j /(n-order+1)) DO
                      Inc(pj);

                    { interpolate linear between tau[pj-1] and tau[pj] }
                    t[order+j]:=tau[pj-1]+(j/(n-order+1)-d[pj-1])/(d[pj]-d[pj-1])*(tau[pj]-tau[pj-1]);
                  END;
                FOR j:=1 TO order DO
                  t[j]:=tau[1];
                FOR j:=n+1 TO n+order DO
                  t[j]:=tau[m];
                FOR j:=1 TO n DO
                  alpha[j]:=0.0;
                Wait(10000);
              END;
              3:BEGIN
                TextBackground(MyBackground);
                ClrScr;
                TextColor(black);
                Msg(10,5,'Set n:=m-order t[j]:= tau[1]                  j= 1.. order');
                Msg(10,7,'               t[j]:= tau[j-order+1]          j=order+1..n');
                Msg(10,9,'               t[j]:= tau[m]                j=n+1..n+order');
                nold:=n;
                n:=m-order;
                IF n > NMAX THEN
                  BEGIN
                    Msg(20,12,'ERROR n > NMAX');wait(5000);
                    n:=nold;
                    Exit;
                  END;
                FOR j:=1 TO order DO
                  t[j]:=tau[1];
                FOR j:=order+1 TO n DO
                  t[j]:=tau[j-order+1];
                FOR j:=n+1 TO n+order DO
                  t[j]:=tau[m];
                FOR j:=1 TO n DO
                  alpha[j]:=0.0;
                Wait(10000);
              END;
            END;
          END;
        END;
      END;
      3:BEGIN
        TextBackground(MyBackground);
        ClrScr;
        Box(20,60,2,4,MyMenuBackground,MyMenuForeground);
        Msg(25,3,'Set indizes of free knots');
        Box(5,75,8,22,Black,White);
        ch:='Y';
        GotoXY(12,10);
        ReadChar('Set all inner knots free ? ',ch);
        IF UpCase(ch) IN ['Y','J'] THEN
          BEGIN
            FOR j:=1 TO l DO
              per[j]:=order+j;
          END
        ELSE
          BEGIN
            FOR j:=1 TO l DO
              BEGIN
                GotoXY(12,10);
                Write('Enter index of ',j:3,' th free knot: ');
                ReadLn(per[j]);
              END;
          END;
        GotoXY(12,10);
        Min_Dist_Knots:=0.0625;
        ReadReal('Enter minimum distance between knots: ',Min_Dist_Knots);
      END;
      4:BEGIN
          Set_Bounds(n,order,t,Active_bounds,lower,upper,TypeOfConstraints);
        END;
    END;
  UNTIL KeyPressed;
END; { BSRead }

{-----------------------------------------------------------------------------}

PROCEDURE Data_To_Screen;
VAR i,j                        : integer;

BEGIN
  TextBackground(MyBackground);
  ClrScr;Box(20,60,1,3,MyMenuBackground,MyMenuForeground);
  Msg(25,2,'Output of data values to screen');
  TextBackground(White);
  TextColor(Black);
  Window(1,5,80,25);j:=1;ClrScr;WriteLn;
  WriteLn('   i !       tau[i]       !         y[i]');
  WriteLn;
  FOR i:=1 TO m DO
    BEGIN
      IF j=15 THEN
        BEGIN
          Wait(200000);ClrScr;WriteLn;j:=1;
          WriteLn('   i !       tau[i]       !         y[i]   ');
          WriteLn;
        END;
      WriteLn(i:4,' !',tau[i]:19,' !',y[i]:19);
      j:=j+1;
    END;
  Wait(200000);
  Window(1,1,80,25);
END;  { Data_to_Screen }

{-----------------------------------------------------------------------------}

PROCEDURE Data_To_Printer;
VAR Headline                   : STRING;
    Year,Month,Day,DayofWeek   : word;
    i                          : integer;

BEGIN
  TextBackground(MyBackground);
  ClrScr;Box(20,60,2,4,MyMenuBackground,MyMenuForeground);
  Msg(25,3,'Output of data values to printer');
  Box(5,75,10,13,MyMenuBackground,MyMenuForeground);
  Msg(7,11,'Enter Headline !');GotoXY(8,12);
  ReadLn(HeadLine);
  Box(20,60,18,20,White,Red);TextColor(Black);
  Msg(21,19,'Press <ENTER> , when printer is ready !');
  REPEAT UNTIL KeyPressed;
  WriteLn(LST,Chr(27)+'@');{ Reset }
  WriteLn(LST,Chr(27)+'<');WriteLn(LST,Chr(27)+'!'+Chr(17));
  WriteLn(LST,' Least squares approximation by B-Splines');
  WriteLn(LST);GetDate(Year,Month,Day,DayofWeek);
  WriteLn(LST,'  ',Headline);
  WriteLn(LST);
  WriteLn(LST,'             Date: ',Day,'.',Month,'.',year);WriteLn(LST);
  WriteLn(LST,Chr(27)+'@',Chr(27)+'!'+Chr(4));
  WriteLn(LST,'         D A T A ');WriteLn(LST);
  WriteLn(LST,'   i !      tau[i]        !        y[i] ');
  WriteLn(lst);
  FOR i:=1 TO m DO
    WriteLn(LST,i:4,' !',tau[i]:19,' !',y[i]:19);
  WriteLn(LST);Wait(10000);
END; { Data_To_Printer }

{-----------------------------------------------------------------------------}

PROCEDURE Data_To_Disk;
VAR i  : integer;
    ch : char;

BEGIN
  TextBackground(MyBackground);
  ClrScr;Box(20,60,2,4,MyMenuBackground,MyMenuForeground);
  Msg(25,3,'Save data values to disk');
  Box(10,70,20,24,MyMenuBackground,MyMenuForeground);
  Msg(15,22,'Enter Filename: ');
  ReadLn(FileName);
  IF FileName=' ' THEN FileName:='TEST.DTA';

  GotoXY(15,23);
  ch:='b';
  ReadChar('Write ascii or binary file (a/b)?: ',ch);

  IF UpCase(ch) <> 'A' THEN
    BEGIN
      Assign(DataFile,FileName);
      Rewrite(DataFile);
      WITH DataRec DO
        BEGIN
          FOR i:=1 TO m DO
            BEGIN
              abscissa:= tau[i];
              ordinate:= y[i];
              Write(DataFile,DataRec);
            END;
        END;
      Close(DataFile);
    END
  ELSE
    BEGIN
      Assign(TxtFile,FileName);
      Rewrite(TxtFile);
      FOR i:=1 TO m DO
        BEGIN
          Write(TxtFile,tau[i]);
          Writeln(TxtFile,y[i]);
        END;
      Close(TxtFile);
    END;
END;  { Data_To_Disk }

{-----------------------------------------------------------------------------}

PROCEDURE mlwritevector(s:STRING; VAR a_addr; n: integer );
{procedure to write vector in Matlab format }
VAR i, nr : integer;
    a     : ARRAY[1..1] OF real ABSOLUTE a_addr;
BEGIN
  WriteLn(TxtFile,'%');
  Write(TxtFile,s,'=[');
  FOR i := 1 TO n DO
    BEGIN
      Write(TxtFile,a[i]);
      IF i < n THEN Write(TxtFile,';');
      IF i MOD 4 = 0 THEN
        IF i < n THEN
          WriteLn(TxtFile);
    END;
  WriteLn(TxtFile,'];');
  WriteLn(TxtFile,'%');
END;

{-----------------------------------------------------------------------------}

PROCEDURE Data_To_Matlab;
VAR i : integer;

BEGIN
  TextBackground(MyBackground);
  ClrScr;Box(20,60,2,4,MyMenuBackground,MyMenuForeground);
  Msg(25,3,'Save data values to m-file');
  Box(10,70,20,24,MyMenuBackground,MyMenuForeground);
  Msg(15,22,'Enter Filename: ');
  ReadLn(FileName);
  IF FileName=' ' THEN FileName:='DATA.M';
  Assign(TxtFile,FileName);
  Rewrite(TxtFile);
  MLWriteVector('x',tau[1],m);
  MLWriteVector('y',y[1],m);
  Close(TxtFile);
END;  { Data_To_Matlab }

{-----------------------------------------------------------------------------}

PROCEDURE Spline_To_Screen;
VAR i,j                        : integer;

BEGIN
  TextBackground(MyBackground);
  ClrScr;Box(20,60,1,3,MyMenuBackground,MyMenuForeground);
  Msg(22,2,'Output of spline parameter to screen');
  TextBackground(White);
  TextColor(Black);
  Window(1,5,80,25);i:=1;ClrScr;WriteLn;
  WriteLn('   j !        t[j]        !       alpha[j]');
  WriteLn;
  FOR j:=1 TO n+order DO
    BEGIN
      IF i=15 THEN
        BEGIN
          Wait(200000);ClrScr;i:=1;WriteLn;
          WriteLn('   j !        t[j]        !       alpha[j]');
          WriteLn;
        END;
      IF j>n THEN
        WriteLn(j:4,' !',t[j]:19,' !')
      ELSE
        WriteLn(j:4,' !',t[j]:19,' !',alpha[j]:19);
      i:=i+1;
    END;
  WriteLn;
  Get_F (m,n,order,t,tau,y,alpha,F);
  RES:=nrm2(m,F,1);
  WriteLn('  Residual sum: ',RES);
  Wait(200000);
  Window(1,1,80,25);
END;  { Spline_to_Screen }

{-----------------------------------------------------------------------------}

PROCEDURE Spline_To_Printer;
VAR Headline                   : STRING;
    Year,Month,Day,DayofWeek   : word;
    j                          : integer;

BEGIN
  TextBackground(MyBackground);
  ClrScr;Box(20,60,2,4,MyMenuBackground,MyMenuForeground);
  Msg(22,3,'Output of spline parameter to printer');
  Box(5,75,10,13,MyMenuBackground,MyMenuForeground);
  Msg(7,11,'Enter Headline !');GotoXY(8,12);
  ReadLn(HeadLine);
  Box(20,60,18,20,White,Red);TextColor(Black);
  Msg(21,19,'Press <ENTER> , when printer is ready !');
  REPEAT UNTIL KeyPressed;
  WriteLn(LST,Chr(27)+'@');{ Reset }
  WriteLn(LST,Chr(27)+'<');WriteLn(LST,Chr(27)+'!'+Chr(17));
  WriteLn(LST,' Least squares approximation by B-Splines');
  WriteLn(LST);GetDate(Year,Month,Day,DayofWeek);
  WriteLn(LST,'  ',Headline);
  WriteLn(LST);
  WriteLn(LST,'             Date: ',Day,'.',Month,'.',year);WriteLn(LST);
  WriteLn(LST,Chr(27)+'@',Chr(27)+'!'+Chr(4));
  WriteLn(LST,'         Spline Parameter ');WriteLn(LST);
  WriteLn(LST,'   j !      t[j]          !        alpha[j]       ');
  WriteLn(lst);
  FOR j:=1 TO n+order DO
    BEGIN
      IF j>n THEN
        WriteLn(LST,j:4,' !',t[j]:19,' !')
      ELSE
        WriteLn(LST,j:4,' !',t[j]:19,' !',alpha[j]:19);
    END;
  WriteLn(LST);
  Get_F (m,n,order,t,tau,y,alpha,F);
  RES:=nrm2(m,F,1);
  WriteLn(LST,'  Residual sum: ',RES);
  WriteLn(LST);
  Wait(10000);
END;  { Spline_To_Printer }

{-----------------------------------------------------------------------------}

PROCEDURE Spline_To_Disk;
VAR j  : integer;
    ch : char;
    h  : real;

BEGIN
  TextBackground(MyBackground);
  ClrScr;Box(20,60,2,4,MyMenuBackground,MyMenuForeground);
  Msg(25,3,'Save spline parameter to disk');
  Box(10,70,20,24,MyMenuBackground,MyMenuForeground);
  Msg(15,22,'Enter Filename: ');
  ReadLn(FileName);
  IF FileName=' ' THEN FileName:='TEST.KNO';

  GotoXY(15,23);
  ch:='b';
  ReadChar('Write ascii or binary file (a/b)?: ',ch);

  IF UpCase(ch) <> 'A' THEN
    BEGIN
      Assign(SplineFile,FileName);
      Rewrite(SplineFile);
      WITH SplineRec DO
        BEGIN
          FOR j:=1 TO n+order DO
            BEGIN
              IF j>n THEN
                BEGIN
                  breakpoint :=t[j];
                  coefficient:=0.0;
                END
              ELSE
                BEGIN
                  breakpoint :=t[j];
                  coefficient:=alpha[j];
                END;
              Write(SplineFile,SplineRec);
            END;
        END;
      Close(SplineFile);
    END
  ELSE
    BEGIN
      Assign(TxtFile,FileName);
      Rewrite(TxtFile);
      FOR j:=1 TO n+order DO
        BEGIN
          IF j>n THEN
            BEGIN
              Write(TxtFile,t[j]);
              h:=0.0;
              Writeln(TxtFile,h);
            END
          ELSE
            BEGIN
              Write(TxtFile,t[j]);
              Writeln(TxtFile,alpha[j]);
            END;
        END;
      Close(TxtFile);
    END;
END;  { Spline_To_Disk }

{-----------------------------------------------------------------------------}

PROCEDURE Spline_To_Matlab;
VAR j : integer;

BEGIN
  TextBackground(MyBackground);
  ClrScr;Box(20,60,2,4,MyMenuBackground,MyMenuForeground);
  Msg(25,3,'Save spline parameter to m-file');
  Box(10,70,20,24,MyMenuBackground,MyMenuForeground);
  Msg(15,22,'Enter Filename: ');
  ReadLn(FileName);
  IF FileName=' ' THEN FileName:='KNOTS.M';
  Assign(TxtFile,FileName);
  Rewrite(TxtFile);
  MLWriteVector('knots',t[1],n+order);
  MLWriteVector('alpha',alpha[1],n);
  Close(TxtFile);
END;  { Spline_To_Matlab }

{-----------------------------------------------------------------------------}

PROCEDURE Statistic_To_Screen;
VAR i,j                        : integer;

BEGIN
  TextBackground(MyBackground);
  ClrScr;Box(20,60,1,3,MyMenuBackground,MyMenuForeground);
  Msg(25,2,'Output of statistic to screen');
  TextBackground(White);
  TextColor(Black);
  Window(1,5,80,25);j:=1;ClrScr;WriteLn;
  WriteLn('   i !    Residual[i]       ');
  WriteLn;
  Get_F (m,n,order,t,tau,y,alpha,F);
  FOR i:=1 TO m DO
    BEGIN
      IF j=15 THEN
        BEGIN
          Wait(200000);ClrScr;WriteLn;j:=1;
          WriteLn('   i !    Residual[i]        ');
          WriteLn;
        END;
      WriteLn(i:4,' !',F[i]:19);
      j:=j+1;
    END;
  WriteLn;
  WriteLn('Norm: ',nrm2(m,F,1));
  IF l<> 0 THEN
    BEGIN
      WriteLn('Indizes of free knots: ');
      FOR i:=1 TO l DO
        Write(per[i]:4);
      WriteLn;
      WriteLn('Minimum Distance between knots: ',Min_Dist_Knots);
    END;
  Wait(200000);
  Window(1,1,80,25);
END;  { Statistic_to_Screen }

{-----------------------------------------------------------------------------}

PROCEDURE Statistic_To_Printer;
VAR Headline                   : STRING;
    Year,Month,Day,DayofWeek   : word;
    i                          : integer;

BEGIN
  TextBackground(MyBackground);
  ClrScr;Box(20,60,2,4,MyMenuBackground,MyMenuForeground);
  Msg(25,3,'Output of statistic to printer');
  Box(5,75,10,13,MyMenuBackground,MyMenuForeground);
  Msg(7,11,'Enter Headline !');GotoXY(8,12);
  ReadLn(HeadLine);
  Box(20,60,18,20,White,Red);TextColor(Black);
  Msg(21,19,'Press <ENTER> , when printer is ready !');
  Get_F (m,n,order,t,tau,y,alpha,F);
  REPEAT UNTIL KeyPressed;
  WriteLn(LST,Chr(27)+'@');{ Reset }
  WriteLn(LST,Chr(27)+'<');WriteLn(LST,Chr(27)+'!'+Chr(17));
  WriteLn(LST,' Least squares approximation by B-Splines');
  WriteLn(LST);GetDate(Year,Month,Day,DayofWeek);
  WriteLn(LST,'  ',Headline);
  WriteLn(LST);
  WriteLn(LST,'             Date: ',Day,'.',Month,'.',year);WriteLn(LST);
  WriteLn(LST,Chr(27)+'@',Chr(27)+'!'+Chr(4));
  WriteLn(LST,'         S T A T I S T I C');WriteLn(LST);
  WriteLn(LST,'   i !      Residual[i]        ');
  WriteLn(lst);
  FOR i:=1 TO m DO
    WriteLn(LST,i:4,' !',F[i]:19);
  WriteLn(LST);
  WriteLn(LST,'Norm: ',nrm2(m,F,1));
  WriteLn(LST);
  IF l<> 0 THEN
    BEGIN
      WriteLn(LST,'Indizes of free knots: ');
      FOR i:=1 TO l DO
        Write(LST,per[i]:4);
      WriteLn(LST);
      WriteLn(LST,'Minimum Distance between knots: ',Min_Dist_Knots);
    END;
  Wait(10000);
END; { Statistic_To_Printer }

{-----------------------------------------------------------------------------}

PROCEDURE BSSave;
VAR snorm : real;
    S     : SmoothingMatrix;
    G     : SmoothingVector;

BEGIN
  REPEAT
    TextBackground(MyBackground);
    ClrScr;MenuLength:=4;
    mm[0,0]:='Output';
    mm[1,0]:='Save data values';
    mm[2,0]:='Save spline parameter';
    mm[3,0]:='Save statistic';
    mm[4,0]:='Smoothing statistic';
    mm[1,1]:='tau[i], y[i]      i=1,..,m ';
    mm[2,1]:='t[j],  j=1,..,n+order;    alpha[j],  j=1,..,n';
    mm[3,1]:='Residual r=y-B*alpha, || r ||, Minimum Distance between knots ';
    mm[4,1]:='Smoothing parameter, || y - B * alpha ||^2 + mu || S * alpha||^2';
    ProgramNo:=Menu(MenuLength,mm,MyMenuBackground,MyMenuForeground);
    CASE ProgramNo OF
      0:Exit;
      1:BEGIN
        TextBackground(MyBackground);
        ClrScr;MenuLength:=4;
        mm[0,0]:='Save Data Values';
        mm[1,0]:='Output to Screen';
        mm[1,1]:='Print data values x[i], y[i] i=1,..,m on screen';
        mm[2,0]:='Output to Printer';
        mm[2,1]:='Print data values x[i], y[i] i=1,..,m';
        mm[3,0]:='Output to Disk';
        mm[3,1]:='Save ascii or binary file to disk';
        mm[4,0]:='Output in Matlab format';
        mm[4,1]:='Save data in m-file to call in Matlab';
        ProgramNo:=Menu(Menulength,mm,MyMenuBackground,MyMenuForeground);
        CASE ProgramNo OF
          0:Exit;
          1:Data_to_Screen;
          2:Data_to_Printer;
          3:Data_to_Disk;
          4:Data_to_Matlab;
        END;
      END;
      2:BEGIN
        TextBackground(MyBackground);
        ClrScr;MenuLength:=4;
        mm[0,0]:='Save Spline Parameter';
        mm[1,0]:='Output to Screen';
        mm[1,1]:='Print knots t[j] and coefficients alpha[j] on screen';
        mm[2,0]:='Output to Printer';
        mm[2,1]:='Print knots t[j] and coefficients alpha[j]';
        mm[3,0]:='Output to Disk';
        mm[3,1]:='Save ascii or binary file to disk';
        mm[4,0]:='Output in Matlab format';
        mm[4,1]:='Save spline parameter in m-file to call in Matlab';
        ProgramNo:=Menu(MenuLength,mm,MyMenuBackground,MyMenuForeground);
        CASE ProgramNo OF
          0:Exit;
          1:Spline_to_Screen;
          2:Spline_to_Printer;
          3:Spline_to_Disk;
          4:Spline_to_Matlab;
        END;
      END;
      3:BEGIN
        TextBackground(MyBackground);
        ClrScr;MenuLength:=2;
        mm[0,0]:='Save Statistic';
        mm[1,0]:='Output to Screen';
        mm[1,1]:='Print residual y[i] - s(x[i]) and free knots on screen';
        mm[2,0]:='Output to Printer';
        mm[2,1]:='Print residual y[i] - s(x[i]) and free knots';
        ProgramNo:=Menu(MenuLength,mm,MyMenuBackground,MyMenuForeground);
        CASE ProgramNo OF
          0:Exit;
          1:Statistic_to_Screen;
          2:Statistic_to_Printer;
        END;
      END;
      4:BEGIN
        TextBackground(MyBackground);
        ClrScr;Box(20,60,2,4,MyMenuBackground,MyMenuForeground);
        Msg(30,3,'Smoothing statistic');
        Box(5,75,8,22,Black,White);

        GotoXY(10,12);
        Write('Order of derivative in smoothing term: ',r:3);

        GotoXY(10,14);
        Write('Smoothing parameter: ',mu:10);

        GotoXY(10,16);
        Compute_Smoothing_Matrix(t,n,order,r,S);
        Get_G(m,n,order,r,mu,t,tau,y,alpha,S,G);
        snorm:=nrm2(m+n-r,G,1);
        Write('Smoothing residual sum: ',snorm);
        GotoXY(10,17);
        Write('(|| y - B * alpha ||^2 + mu || S * alpha ||^2)^(1/2)');
        Wait(20000);
      END;
    END;
  UNTIL KeyPressed;
END;  { BSSave }

{-----------------------------------------------------------------------------}

PROCEDURE Correct_Data;

VAR i : integer;
   ch : char;

BEGIN
  TextBackground(MyBackground);
  ClrScr;Box(20,60,2,4,MyMenuBackground,MyMenuForeground);
  Msg(30,3,'Correct data values');
  REPEAT
    Box(5,75,8,22,Black,White);
    Msg(10,12,'Enter number of element to correct (0 to exit) :            ');
    GotoXY(59,12);
    ReadLn(i);
    IF (i<=m) AND (i>=1) THEN
      BEGIN
        msg(10,14,'Press <ENTER> to continue');

        GotoXY(10,16);
        ReadReal('tau['+Int2Str(i)+']= ',tau[i]);

        GotoXY(10,18);
        ReadReal('y['+Int2Str(i)+']= ',y[i]);

      END
    ELSE Write(Chr(7));
  UNTIL i=0;
END; { Correct_Data }

{-----------------------------------------------------------------------------}

PROCEDURE Correct_Spline;

VAR j : integer;
   ch : char;

BEGIN
  TextBackground(MyBackground);
  ClrScr;Box(20,60,2,4,MyMenuBackground,MyMenuForeground);
  Msg(27,3,'Correct spline parameter');
  REPEAT
    Box(5,75,8,22,Black,White);
    Msg(10,12,'Enter number of element to correct (0 to exit) :            ');
    GotoXY(62,12);
    ReadLn(j);
    IF (j<=n+order) AND (j>=1) THEN
      BEGIN
        msg(10,14,'Press <ENTER> to continue');

        GotoXY(10,16);
        IF j>n THEN
          ReadReal('t['+Int2Str(j)+']= ',t[j])
        ELSE
          BEGIN
            ReadReal('t['+Int2Str(j)+']= ',t[j]);
            GotoXY(10,20);
            ReadReal('alpha['+Int2Str(j)+']= ',alpha[j]);
          END;
      END
    ELSE Write(Chr(7));
  UNTIL j=0;
END; { Correct_Spline }

{-----------------------------------------------------------------------------}

PROCEDURE Knot_Condensing(VAR t : knotsequence;VAR n,k,l: integer);
{ Reduce number of knots or set new knots if constraints are activ }

VAR i,j,r,choice,choice1, nold, Multiplicity  : integer;
    average_t                                 : real;

BEGIN
  TextBackground(MyBackground);
  ClrScr;Box(20,60,2,4,MyMenuBackground,MyMenuForeground);
  Msg(30,3,'Condense spline knots');

  nold:=n;
  i:=k;
  REPEAT
    { work on knot i }
    j:=1;
    { Constraints almost active or relative error in knots < 0.2 % }
    WHILE (t[i+j]-t[i+j-1] <= Min_Dist_Knots*2.0) OR
          ( Abs(t[i+j]-t[i+j-1]) < 0.002* Abs(t[i+j])) DO
      j:=j+1;
    IF j>1 THEN
      BEGIN
        Box(5,75,7,21,Black,White);
        Msg(9,9,'Constraints for following knots almost active ');
        FOR r:=0 TO j-1 DO
          BEGIN
            GotoXY(9,11+r);
            ReadReal('t['+Int2Str(i+r)+'] : ',t[i+r]);
          END;

        Msg(9,19,'1 -- one simple knot, 2 -- multiple knot, 3 -- do nothing');
        Msg(9,20,'Choice: ');
        ReadLn(Choice);

        IF (choice=1) OR (choice=2) THEN
          BEGIN
            GotoXY(9,19);
            ReadInt('Number of knots to include: ',j);
            Msg(9,19,'1 -- Knot equal left knot, 2 -- Knot equal average of knots');
            Msg(9,20,'Choice: ');
            ReadLn(choice1);
          END;
        IF choice=2 THEN
          BEGIN
            Msg(9,20,' Multiplicity of new knot: ');
            GotoXY(36,20);Write(j);
            GotoXY(36,20);
            ReadLn(Multiplicity);
          END;

        average_t:=0.0;
        FOR r:=0 TO j-1 DO
          average_t:=average_t+t[i+r];
        average_t:=average_t/j;
        IF choice1=1 THEN average_t:=t[i];

        CASE choice OF
          1 : BEGIN
            t[i]:=average_t;
            FOR r:=i+1 TO n+k DO
              t[r]:=t[r+j-1];
            n:=n-j+1;
            i:=i+j-1;
          END;
          2 : BEGIN
            FOR r:=0 TO multiplicity-1 DO
              t[i+r]:=average_t;      { set knot t[i] of multiplicity }
            FOR r:=i+multiplicity TO n+k DO
              t[r]:=t[r+j-1];
            n:=n-j+1;
            i:=i+j-1;
          END
          ELSE
            BEGIN
              i:=i+j-1;
              { do nothing }
            END;
        END;
      END
    ELSE i:=i+1;
  UNTIL i+j>n;
  IF n<> nold THEN
    BEGIN
      Box(5,75,22,24,White,Black);
      TextColor(red);
      Write(Chr(7));
      Msg(20,23,'Change number and indizes of free knots');
      l:=0;
      wait(2000);
    END;
END; { Knot_Condensing }

{-----------------------------------------------------------------------------}

PROCEDURE Knot_Removing(VAR t : knotsequence;VAR i,n,k,l: integer);
{ Remove knots beginning from t[i] }

VAR j,r,choice,choice1, nold, Multiplicity  : integer;
    average_t                               : real;

BEGIN
  TextBackground(MyBackground);
  ClrScr;Box(20,60,2,4,MyMenuBackground,MyMenuForeground);
  Msg(30,3,'Remove spline knots');

  nold:=n;

  j:=6;
  IF i+j > n+1 THEN
    j:=n-i+1;

  Box(5,75,7,21,Black,White);
  FOR r:=0 TO j-1 DO
    BEGIN
      GotoXY(9,11+r);
      ReadReal('t['+Int2Str(i+r)+'] : ',t[i+r]);
    END;



  Msg(9,19,'1 -- one simple knot, 2 -- multiple knot, 3 -- do nothing');
  Msg(9,20,'Choice: ');
  ReadLn(Choice);
  IF (choice=1) OR (choice=2) THEN
    BEGIN
      GotoXY(9,19);
      ReadInt('Number of knots to include: ',j);
      Msg(9,19,'1 -- Knot equal left knot, 2 -- Knot equal average of knots');
      Msg(9,20,'Choice: ');
      ReadLn(choice1);
    END;
  IF (choice=2) THEN
    BEGIN
      Msg(9,20,' Multiplicity of new knot: ');
      GotoXY(36,20);Write(j);
      GotoXY(36,20);
      ReadLn(Multiplicity);
    END;

  average_t:=0.0;
  FOR r:=0 TO j-1 DO
    average_t:=average_t+t[i+r];
    average_t:=average_t/j;
  IF choice1=1 THEN average_t:=t[i];

  CASE choice OF
    1:BEGIN
      t[i]:=average_t;
      FOR r:=i+1 TO n+k DO
        t[r]:=t[r+j-1];
      n:=n-j+1;
      i:=i+j-1;
    END;
    2:BEGIN
      FOR r:=0 TO multiplicity-1 DO
        t[i+r]:=average_t;
      FOR r:=i+multiplicity TO n+k DO
        t[r]:=t[r+j-1];
      n:=n-j+1;
      i:=i+j-1;
    END
    ELSE
      BEGIN
        i:=i+j-1; { do nothing }
      END;
  END;
  IF n<> nold THEN
    BEGIN
      Box(5,75,22,24,White,Black);
      TextColor(red);
      Write(Chr(7));
      Msg(20,23,'Change number and indizes of free knots');
      l:=0;
      wait(2000);
    END;
END; { Knot_Removing }

{-----------------------------------------------------------------------------}

PROCEDURE Correct;
{ correct data }
VAR i        : integer;

BEGIN
  TextBackground(MyBackground);
  ClrScr;MenuLength:=4;
  mm[0,0]:='Correct';
  mm[1,0]:='Correct data values';
  mm[2,0]:='Correct spline parameter';
  mm[3,0]:='Condense spline knots';
  mm[4,0]:='Remove spline knots';
  mm[1,1]:='Correct tau[i], y[i], w[i],    i=1,..,m ';
  mm[2,1]:='Correct t[j], j= 1,..,n+order;  alpha[j], j=1,..,n';
  mm[3,1]:='Reduce number of knots or set multiple knots for active constraints';
  mm[4,1]:='Remove knots or set new knots for a specific point';
  ProgramNo:=Menu(MenuLength,mm,MyMenuBackground,MyMenuForeground);
  CASE ProgramNo OF
    0:Exit;
    1:Correct_Data;
    2:Correct_Spline;
    3:Knot_Condensing(t,n,order,l);
    4:BEGIN
      Box(20,60,6,8,MyMenuBackground,MyMenuForeground);
      Msg(23,7,' Enter number of knot: ');
      ReadLn(i);
      Knot_Removing(t,i,n,order,l);
    END;
  END;
END; { Correct }

{-----------------------------------------------------------------------------}

PROCEDURE Parameter;
{ Change number of data values and dimension of spline space }
VAR ch : char;

BEGIN
  TextBackground(MyBackground);
  ClrScr;Box(20,60,2,4,MyMenuBackground,MyMenuForeground);
  Msg(30,3,'Change Parameter');
  Box(5,75,8,22,Black,White);
  Msg(15,9,'Enter a new parameter or press <ENTER>');
  GotoXY(10,11);
  ReadInt('Number of data points m (<='+Int2Str(MMAX)+'): ',m);
  GotoXY(10,13);
  ReadInt('Dimension of spline space n (<='+Int2Str(NMAX)+'): ',n);
  GotoXY(10,15);
  ReadInt('Order of B-Splines (<='+Int2Str(NBANDS)+'): ',order);
  GotoXY(10,17);
  ReadInt('Number of free knots (<='+Int2Str(n-order)+'): ',l);
  GotoXY(10,19);
  ReadInt('Order of derivative in smoothing term (<'+Int2Str(order)+'): ',r);
  GotoXY(10,21);
  ReadReal('Smoothing Parameter: ',mu);
END; { Parameter }

{-----------------------------------------------------------------------------}

END.