PROGRAM free;
{ Least Squares Approximation by B-splines with free knots

  Copyright (c) 1993-1996, 1997 by T. Schuetze
  All rights reserved.

  Your are permitted to use, copy, modify, and distribute this software and
  its documentation in source and binary forms, provided that:

  i)  any distribution of the software is on the same terms as in this
      License Agreement, and each copy contains the copyright notice that
      appears in this software; and

  ii) any documentation or other materials related to such distribution or
      use acknowledge that the software was developed by T. Schuetze.


  The primary purpose of this software is to provide a convenient framework
  for the numerical tests of the methods developed in

    H. Schwetlick and T. Schuetze, Least squares approximation by splines
    with free knots, BIT 35 (1995), no. 3, 361--384

    T. Schuetze and H. Schwetlick, Constrained approximation by splines with
    free knots, Preprint MATH-NM-18-1995, TU Dresden, 1995 (submitted to BIT)

    T. Schuetze and H. Schwetlick, Constrained approximation by splines with
    free knots, BIT 37 (1997), no. 1, 105--137

  A manual describing the main features of the program is available as

    T. Schuetze, FREE -- A program for constrained approximation by splines
    with free knots, Preprint MATH-NM-04-1996, TU Dresden, 1996

  Research of T. Schuetze was supported by Deutsche Forschungsgemeinschaft
  under grant Schm 968/2-1,2-2.

  Dresden, February 1996, November 1997

  Torsten Schuetze
  Technical University of Dresden
  Department of Mathematics
  Institute of Numerical Mathematics
  D-01062 Dresden, Germany
  E-mail: schuetze@math.tu-dresden.de
  WWW: http://www.math.tu-dresden.de/~schuetze                                }

{*****************************************************************************}

{ FREE was developed with Borland Pascal 7.0, but it should be working with
  any version of Turbo-Pascal superior version 5.5.                           }

{ We need a lot of variables. Therefore we must increase the stack size to
  its maximum value                                                           }
{$M 65520,0,655360}

{$define sound}
{$undef sound}             { We don't want to hear the beep.}
{$define mixed} {backtracking with mixed quadratic/cubic polynomials}

USES def,         { definitions of constants, types and global variables      }
     error,       { error handler                                             }
     various,     { various procedures for menu handling                      }
     graphic,     { graphical presentation of resulting spline                }
     InOut,       { procedures for in/output of the program                   }
     BSpline,     { procedures to compute B-Splines and l2-approximations     }
     Smooth,      { procedures to compute smoothing B-Splines                 }
     CSspline,    { procedures for constrained smoothing B-Splines            }
     Knotrem,     { automatic knot removal procedures                         }
     Gauss,       { Gauss-Newton-like methods for nonlinear problems          }
     BLAS,        { Basic Linear Algebra Subroutines                          }
     crt;


VAR ch          : char;
    OldExitProc : Pointer;          { Saves exit procedure address }
    B           : LeastSquareMatrix;
    Q           : OrthogonalMatrix;
    S           : SmoothingMatrix;  { Smoothing Matrix             }
    i,j,time_no : integer;
    tsave       : knotsequence;

{$F+}
PROCEDURE MyExitProc;
BEGIN
  ExitProc := OldExitProc; { Restore exit procedure address }
  IF (ExitCode<>0) AND (NOT Assigned(ErrorAddr)) THEN
    Error_Handler('Run time error in FREE');
END; { MyExitProc }
{$F-}


BEGIN  { mainprogram }
  time_no:=100;
  OldExitProc := ExitProc;                { save previous exit proc }
  ExitProc := @MyExitProc;                { insert our exit proc in chain }
  REPEAT
    TextBackground(MyBackGround);ClrScr;
    Box(10,70,2,5,MyMenuBackground,MyMenuForeground);
    Msg(12,3,'Least squares approximation by B-Splines with free knots');
    Msg(40,4,'by Torsten Schtze');
    Menulength:=8;
    mm[0,0]:='Main Menu';
    mm[1,0]:='Parameter';
    mm[1,1]:='Set number of data values, spline breakpoints, free knots and order';
    mm[2,0]:='Load';
    mm[2,1]:='Load data values and spline parameter, set constraints';
    mm[3,0]:='Save';
    mm[3,1]:='Save data values and spline parameter';
    mm[4,0]:='Correct';
    mm[4,1]:='Correct data values and spline parameter, reduce number of knots';
    mm[5,0]:='Approximation';
    mm[5,1]:='Linear least squares approximation';
    mm[6,0]:='Free knots';
    mm[6,1]:='Nonlinear least squares approximation with free knots';
    mm[7,0]:='Knot removal';
    mm[7,1]:='Procedures for removing knots';
    mm[8,0]:='Graphic';
    mm[8,1]:='Graphical presentation of the resulting spline';
    ProgramNo:=Menu(MenuLength,mm,MyMenuBackground,MyMenuForeground);
    CASE ProgramNo OF
      1: Parameter;
      2: BSRead;
      3: BSSave;
      4: Correct;
      5: BEGIN
        TextBackground(MyBackground);
        ClrScr;
        MenuLength:=4;
        mm[0,0]:='Approximation';
        mm[1,0]:='Approximation';
        mm[2,0]:='Smoothing';
        mm[3,0]:='Constrained Approximation';
        mm[4,0]:='Constrained Smoothing';
        mm[1,1]:='Compute l_2-Approximation with fixed knots';
        mm[2,1]:='Compute smoothing spline with fixed knots';
        mm[3,1]:='Compute constrained spline with fixed knots';
        mm[4,1]:='Compute constrained smoothing spline with fixed knots';
        ProgramNo:=Menu(MenuLength,mm,MyMenuBackground,MyMenuForeground);

        CASE ProgramNo OF
          1: BEGIN
            Box(20,60,5,7,White,Black);
            TextColor(Red);
            Msg(22,6,'Computing coefficients. Please wait!');
            l2Appr(m,n,order,t,tau,y,alpha,RES);
            {$ifdef sound}
            Write(Chr(7),Chr(7));
            {$endif}
          END;
          2: BEGIN
            Box(20,60,5,7,White,Black);
            TextColor(Red);
            Msg(22,6,'Computing coefficients. Please wait!');
            Compute_Smoothing_Matrix(t,n,order,r,S);
            Compute_Smoothing_spline(m,n,order,t,r,mu,tau,y,S,alpha,RES);
            {$ifdef sound}
            Write(Chr(7),Chr(7));
            {$endif}
          END;
          3: BEGIN
            Box(20,60,5,7,White,Black);
            TextColor(Red);
            Msg(22,6,'Computing coefficients. Please wait!');
            Constrained_Spline(m,n,order,t,active_bounds,lower,upper,
            TypeOfConstraints,tau,y,alpha,RES,false,false,B,Q);
            {$ifdef sound}
            Write(Chr(7),Chr(7));
            {$endif}
          END;
          4: BEGIN
            Box(20,60,5,7,White,Black);
            TextColor(Red);
            Msg(22,6,'Computing coefficients. Please wait!');
            Compute_Smoothing_Matrix(t,n,order,r,S);
            Constrained_Smoothing_Spline(m,n,order,t,r,mu,
               active_bounds,lower,upper,TypeOfConstraints,tau,y,alpha,S,RES,false,false,B,Q);
            {$ifdef sound}
            Write(Chr(7),Chr(7));
            {$endif}
          END;
        END;
      END;
      6: BEGIN
        TextBackground(MyBackground);
        ClrScr;
        MenuLength:=5;
        mm[0,0]:='Free Knots';
        mm[1,0]:='Approximation';
        mm[2,0]:='Smoothing';
        mm[3,0]:='Constrained Approximation';
        mm[4,0]:='Constrained Smoothing';
        mm[5,0]:='Set parameter';
        mm[1,1]:='l_2-approximation by splines with free knots (no smoothing)';
        mm[2,1]:='l_2-approximation by smoothing splines with free knots';
        mm[3,1]:='l_2-approximation by constrained splines with free knots (no smoothing)';
        mm[4,1]:='l_2-approximation by constrained smoothing splines with free knots';
        mm[5,1]:='Set termination parameter for Gauss-Newton methods';
        ProgramNo:=Menu(MenuLength,mm,MyMenuBackground,MyMenuForeground);
        CASE ProgramNo OF
          0: Exit;
          1: BEGIN
            Box(20,60,5,7,White,Black);
            Msg(23,6,' Kaufman Model (y/n): ');
            ReadLn(ch);
            IF UpCase(ch) IN ['Y','J'] THEN
              Kaufman_Model:=TRUE
            ELSE
              Kaufman_Model:=FALSE;
            Box(20,60,5,7,White,Black);
            Msg(23,6,' rel. Constraints (y/n): ');
            ReadLn(ch);
            IF UpCase(ch) IN ['Y','J'] THEN
              Constraints_abs:=FALSE
            ELSE
              Constraints_abs:=TRUE;
            Box(20,60,5,7,White,Black);
            TextColor(Red);
            Msg(23,6,'Computing free knots. Please wait!');
            step:=STEPMAX;
            start:=StopTime;
            { Zeitmessung }
            for j:=1 to n+order do
              tsave[j]:=t[j];
            for i:=1 to time_no do
              begin
                for j:=1 to n+order do
                  t[j]:=tsave[j];
                Regularized_Generalized_Gauss_Newton(m,n,order,t,l,per,step,
                RetCode,normGradient,Grad_p);
              end;
            { Zeitmessung }
            Stop:=StopTime;
            {$ifdef sound}
            Write(Chr(7),Chr(7));
            {$endif}
            Box(20,60,5,7,White,Black);
            Msg(23,6,'Steps: ');
            Write(step);
            Write('  Time: ',Stop-Start:5:0);
            Write(' RetCode: ',RetCode:3);
            Box(20,60,1,4,White,Black);
            Msg(22,2,'|F^T * J * s|: ');
            Write(Abs(grad_p));
            Msg(22,3,'Norm Gradient: ');
            Write(normGradient);
            Wait(20000);
          END;
          2: BEGIN
            Box(20,60,5,7,White,Black);
            Msg(23,6,' Kaufman Model (y/n): ');
            ReadLn(ch);
            IF UpCase(ch) IN ['Y','J'] THEN
              Kaufman_Model:=TRUE
            ELSE
              Kaufman_Model:=FALSE;
            Box(20,60,5,7,White,Black);
            Msg(23,6,' rel. Constraints (y/n): ');
            ReadLn(ch);
            IF UpCase(ch) IN ['Y','J'] THEN
              Constraints_abs:=FALSE
            ELSE
              Constraints_abs:=TRUE;
            Box(20,60,5,7,White,Black);
            TextColor(Red);
            Msg(23,6,'Computing free knots. Please wait!');
            step:=STEPMAX;
            start:=StopTime;
            { Zeitmessung }
            for j:=1 to n+order do
              tsave[j]:=t[j];
            for i:=1 to time_no do
              begin
                for j:=1 to n+order do
                  t[j]:=tsave[j];
                Smoothing_Regularized_Generalized_Gauss_Newton(
                 m,n,order,t,r,mu,l,per,step,RetCode,normGradient,Grad_p);
              end;
            { Zeitmessung }
            Stop:=StopTime;
            {$ifdef sound}
            Write(Chr(7),Chr(7));
            {$endif}
            Box(20,60,5,7,White,Black);
            Msg(23,6,'Steps: ');
            Write(step);
            Write('  Time: ',Stop-Start:5:0);
            Write(' RetCode: ',RetCode:3);
            Box(20,60,1,4,White,Black);
            Msg(22,2,'|F^T * J * s|: ');
            Write(Abs(grad_p));
            Msg(22,3,'Norm Gradient: ');
            Write(normGradient);
            Wait(20000);
          END;
          3: BEGIN
            Box(20,60,5,7,White,Black);
            Msg(23,6,' Kaufman Model (y/n): ');
            ReadLn(ch);
            IF UpCase(ch) IN ['Y','J'] THEN
              Kaufman_Model:=TRUE
            ELSE
              Kaufman_Model:=FALSE;
            Box(20,60,5,7,White,Black);
            Msg(23,6,' rel. Constraints (y/n): ');
            ReadLn(ch);
            IF UpCase(ch) IN ['Y','J'] THEN
              Constraints_abs:=FALSE
            ELSE
              Constraints_abs:=TRUE;
            Box(20,60,5,7,White,Black);
            TextColor(Red);
            Msg(23,6,'Computing free knots. Please wait!');
            step:=STEPMAX;
            start:=StopTime;
            { Zeitmessung }
            for j:=1 to n+order do
              tsave[j]:=t[j];
            for i:=1 to time_no do
              begin
                for j:=1 to n+order do
                  t[j]:=tsave[j];
                  Constrained_Regularized_Generalized_Gauss_Newton(
                    m,n,order,t,active_bounds,lower,upper,
                    TypeOfConstraints,l,per,step,RetCode,normGradient,Grad_p);
              end;
            { Zeitmessung }
            Stop:=StopTime;
            {$ifdef sound}
            Write(Chr(7),Chr(7));
            {$endif}
            Box(20,60,5,7,White,Black);
            Msg(23,6,'Steps: ');
            Write(step);
            Write('  Time: ',Stop-Start:5:0);
            Write(' RetCode: ',RetCode:3);
            Box(20,60,1,4,White,Black);
            Msg(22,2,'|F^T * J * s|: ');
            Write(Abs(grad_p));
            Msg(22,3,'Norm Gradient: ');
            Write(normGradient);
            Wait(20000);
          END;
          4: BEGIN
            Box(20,60,5,7,White,Black);
            Msg(23,6,' Kaufman Model (y/n): ');
            ReadLn(ch);
            IF UpCase(ch) IN ['Y','J'] THEN
              Kaufman_Model:=TRUE
            ELSE
              Kaufman_Model:=FALSE;
            Box(20,60,5,7,White,Black);
            Msg(23,6,' rel. Constraints (y/n): ');
            ReadLn(ch);
            IF UpCase(ch) IN ['Y','J'] THEN
              Constraints_abs:=FALSE
            ELSE
              Constraints_abs:=TRUE;
            Box(20,60,5,7,White,Black);
            TextColor(Red);
            Msg(23,6,'Computing free knots. Please wait!');
            step:=STEPMAX;
            start:=StopTime;
            { Zeitmessung }
            for j:=1 to n+order do
              tsave[j]:=t[j];
            for i:=1 to time_no do
              begin
                for j:=1 to n+order do
                  t[j]:=tsave[j];
                  Constrained_Smoothing_Regularized_Generalized_Gauss_Newton(
                    m,n,order,t,r,mu,active_bounds,lower,upper,
                    TypeOfConstraints,l,per,step,RetCode,normGradient,Grad_p);
              end;
            { Zeitmessung }
            Stop:=StopTime;
            {$ifdef sound}
            Write(Chr(7),Chr(7));
            {$endif}
            Box(20,60,5,7,White,Black);
            Msg(23,6,'Steps: ');
            Write(step);
            Write('  Time: ',Stop-Start:5:0);
            Write(' RetCode: ',RetCode:3);
            Box(20,60,1,4,White,Black);
            Msg(22,2,'|F^T * J * s|: ');
            Write(Abs(grad_p));
            Msg(22,3,'Norm Gradient: ');
            Write(normGradient);
            Wait(20000);
          END;
          5: BEGIN
            TextBackground(MyBackground);
            ClrScr;
            Box(20,60,2,4,MyMenuBackground,MyMenuForeground);
            Msg(30,3,'Set Parameter');
            Box(5,75,8,23,Black,White);
            Msg(15,10,'Enter a new parameter or press <ENTER>');
            GotoXY(9,12);
            ReadReal('Bound for Defect of || F ||           : ',EPS0);
            GotoXY(9,14);
            ReadReal('Bound for relative change of || x^k ||: ',EPS1);
            GotoXY(9,16);
            ReadReal('Bound for relative change of || F^k ||: ',EPS3);
            GotoXY(9,18);
            ReadReal('Bound for norm of Gradient || J^T F ||: ',EPSGrad);
            GotoXY(9,20);
            ReadReal('Bound for | J^T F s |                 : ',EPSDesc);
            GotoXY(9,22);
            ReadInt('Max. number of Gauss-Newton steps     : ',STEPMAX);
          END;
        END;
      END;
      7: BEGIN
        TextBackground(MyBackground);
        ClrScr;
        MenuLength:=3;
        mm[0,0]:='Knot removal';
        mm[1,0]:='Phase 1';
        mm[2,0]:='Phase 2';
        mm[3,0]:='Remove one knot';
        mm[1,1]:='Automatic knot removal procedure without relocating knots';
        mm[2,1]:='Automatic knot removal procedure with relocating knots';
        mm[3,1]:='Remove knot with smallest jump in (k-1)-th derivative';
        ProgramNo:=Menu(MenuLength,mm,MyMenuBackground,MyMenuForeground);
        CASE ProgramNo OF
          1: BEGIN
            Automatic_Knot_Removal_Phase1;
          END;
          2: BEGIN
            Automatic_Knot_Removal_Phase2;
          END;
          3: BEGIN
            Remove_Knot;
          END;
        END;
      END;
      8: graphik;
      0: BEGIN
        TextBackground(MyBackground);ClrScr;
        Box(20,65,10,14,MyMenuBackground,MyMenuForeground);
        Msg(22,11,'Do you really wish to exit this program?');
        msg(35,12,'Y  Yes');
        msg(35,13,'N  No');
        ch:=ReadKey;ch:=UpCase(ch);
        IF ch='Y' THEN
          BEGIN
            TextBackground(Black);
            ClrScr;
            Exit;
          END;
      END;
    END;
  UNTIL false;
END. { FREE }
