UNIT Various;
{ Unit with various procedures for menu handling                              }

INTERFACE

USES crt,dos,     { Standard Turbo-Pascal Units                               }
     def;         { definitions of constants, types and global variables      }

CONST Verbose                          :BOOLEAN=true;
     { if verbose=true then draw help menu          }

TYPE MenuArray=ARRAY[0..9,0..1] OF STRING;
     { contains max. 9 menuitems with help text     }

VAR  MenuLength,TxtColor,BkColor       :WORD;
     mm                                :Menuarray;

{-----------------------------------------------------------------------------}

FUNCTION StopTime : double;
{ System time in 1/100 sec }

{-----------------------------------------------------------------------------}

PROCEDURE Msg(X,Y:integer;S:STRING);
{ Write the message S at X,Y }

{-----------------------------------------------------------------------------}

PROCEDURE Box(Xmin,Xmax,Ymin,Ymax,BkColor,TxtColor:integer);
{ Draw a Rectangle with Borders }

{-----------------------------------------------------------------------------}

PROCEDURE ReadReal( Str:STRING; VAR x:real);
  { Read real number from terminal }

{-----------------------------------------------------------------------------}

PROCEDURE ReadInt( Str:STRING; VAR i:integer);
  { Read integer from terminal }

{-----------------------------------------------------------------------------}

PROCEDURE ReadChar( Str:STRING; VAR ch:char);
  { Read character from terminal }

{-----------------------------------------------------------------------------}

FUNCTION Int2Str(I: Longint): STRING;
{ Convert any integer type to a string }

{-----------------------------------------------------------------------------}

PROCEDURE WriteSlow(S:STRING;Time:Integer);
{ Write the string S slow }

{-----------------------------------------------------------------------------}

PROCEDURE Wait(Time:double);
{ Wait until RETURN or n=>Time }

{-----------------------------------------------------------------------------}

PROCEDURE HardcopyON;
{ Interrupt $05 , Hardcopy to Printer }

{-----------------------------------------------------------------------------}

FUNCTION MENU(MenuLength:WORD;VAR mm:menuarray;BKColor,TxtColor:WORD):INTEGER;
{ Draw Menu on Screen and Select Menuitem }

IMPLEMENTATION

{-----------------------------------------------------------------------------}

FUNCTION StopTime : double;
{ System time in 1/100 sec }
VAR hour,min,sec,sec100 : word;
BEGIN
  GetTime(hour,min,sec,sec100);
  StopTime:=sec100+100.0*sec+6000.0*min+360000.0*hour;
END;  { StopTime }

{-----------------------------------------------------------------------------}

PROCEDURE Msg(X,Y:integer;S:STRING);
BEGIN  GotoXY(X,Y);Write(s);
END;  { Msg }

{-----------------------------------------------------------------------------}

PROCEDURE Box(Xmin,Xmax,Ymin,Ymax,BkColor,TxtColor:integer);
VAR i:integer;
BEGIN
  TextBackground(BkColor);TextColor(TxtColor);
  FOR i:=Xmin+1 TO Xmax-1 DO
    BEGIN
      Msg(i,ymin,#205);
      Msg(i,ymax,#205);
    END;
  FOR i:=Ymin+1 TO Ymax-1 DO
    BEGIN Msg(Xmin,i,#186);Write('':(Xmax-Xmin));
      Msg(Xmax,i,#186);
    END;
  Msg(Xmin,Ymax,#200);Msg(Xmin,Ymin,#201);
  Msg(Xmax,Ymax,#188);Msg(Xmax,Ymin,#187);
END;  { Box }

{-----------------------------------------------------------------------------}

PROCEDURE ReadReal( Str:STRING; VAR x:real);
LABEL 1;
VAR c            : char;
    s            : STRING;
    mode,wx1,wx2 : integer;
    y            : real;
BEGIN
  Write(Str);
  wx1 := WhereX;
  Write( x);
  wx2 := WhereX;
  GotoXY(wx1,WhereY);
1:
  c := ReadKey;
  IF c <> Chr(13) THEN
    BEGIN
      Write('':wx2-wx1+1);
      GotoXY( wx1, WhereY);
      Write(c);
      ReadLn(s);
      wx2 := WhereX;
      s := c + s;
      Val( s, y, mode);
      IF mode <> 0 THEN
        BEGIN
          GotoXY(wx1,WhereY);Write('':wx2-wx1+1);
          GotoXY(wx1,WhereY);
          GOTO 1
        END
      ELSE
        x := y;
    END{ if c}
END;

{-----------------------------------------------------------------------------}

PROCEDURE ReadInt( Str:STRING; VAR i:integer);
LABEL 1;
VAR c       : char;
    s       : STRING;
    j,mode  : integer;
    wx1,wx2 : integer;
BEGIN
  Write( Str);
  wx1 := WhereX;
  Write(i);
  wx2 := WhereX;
  GotoXY(wx1,WhereY);
1:
  c := ReadKey;
  IF c <> Chr(13) THEN
    BEGIN
      Write('':wx2-wx1+1);GotoXY(wx1,WhereY);
      Write(c);ReadLn(s); wx2 := WhereX;
      s := c + s;
      Val( s, j, mode);
      IF mode <> 0 THEN
        BEGIN
          GotoXY(wx1,WhereY);
          Write('':wx2-wx1+1);
          GotoXY(wx1,WhereY);
          GOTO 1
        END
      ELSE
        i := j;
    END{ if c}
END;

{-----------------------------------------------------------------------------}

PROCEDURE ReadChar( Str:STRING; VAR ch:char);
VAR c:char;
BEGIN
  Write( Str);
  Write( ch);
  GotoXY( WhereX-1,WhereY);
  c := ReadKey;
  IF c <> Chr(13) THEN
    ch := c;
  Write( ch );
END;

{-----------------------------------------------------------------------------}

FUNCTION Int2Str(I: Longint): STRING;
{ Convert any integer type to a string }
VAR
 S: STRING[11];
BEGIN
 Str(I, S);
 Int2Str := S;
END;

{-----------------------------------------------------------------------------}

PROCEDURE WriteSlow(S:STRING;Time:Integer);
VAR i:integer;
BEGIN
  FOR i:=1 TO Length(s) DO
    BEGIN Delay(Time);Write(s[i]);
    END;
END; { WriteSlow }

{-----------------------------------------------------------------------------}

PROCEDURE Wait(Time:double);
VAR n:double;
    quit:boolean;
BEGIN
  n:=1;
  REPEAT  n:=n+1;Quit:=(ReadKey=^M);
  UNTIL (n>Time) OR quit;
END; { Wait }

{-----------------------------------------------------------------------------}

PROCEDURE HardcopyON;
VAR   regi : REGISTERS;
CONST  BIOS_PB_SCR : ARRAY[1..5] OF byte = (22,$10,0,0,0);
BEGIN
  regi.DX := Ofs(BIOS_PB_SCR);
  regi.CX := $32; { direct BIOS-CALL }
  regi.DS := Seg(BIOS_PB_SCR);
  Intr($05,regi);
END; { HardcopyON }

{-----------------------------------------------------------------------------}

PROCEDURE readkbd(VAR a:CHAR);
BEGIN
  a:=ReadKey;
  IF a=#0 THEN
    BEGIN
      a:=ReadKey;
      CASE a OF
        ';':a:='?';
        'G':a:=^A;
        'H':a:=^E;
        'I':a:=^R;
        'K':a:=^S;
        'M':a:=^D;
        'O':a:=^F;
        'P':a:=^X;
        'Q':a:=^C;
        'R':a:=^V;
        'S':a:=^H;
        Chr(72):a:=^E;
        Chr(80):a:=^X;
      END;
    END;
END;     { ReadKbd }

{-----------------------------------------------------------------------------}

FUNCTION MENU(MenuLength:WORD;VAR mm:menuarray;BkColor,TxtColor:WORD):INTEGER;
CONST Xmin=20;
      Xmax=60;
      Ymin=8;

VAR ToC : STRING[2];
    ch  : char;

PROCEDURE DrawBorder;

VAR i:integer;

BEGIN
  TextBackground(BkColor);TextColor(TxtColor);
  FOR i:=Xmin+1 TO Xmax-1 DO
    BEGIN
      Msg(i,Ymin,#205);Msg(i,Ymin+2,#205);
      Msg(i,Ymin+Menulength+3,#205);
      Msg(i,Ymin+Menulength+5,#205);
    END;
  Msg(Xmin,Ymin,#201);Msg(Xmax,Ymin,#187);
  Msg(Xmin,Ymin+1,#186);Write('':(Xmax-Xmin-1),#186);
  Msg(Xmin,Ymin+2,#204);Msg(Xmax,Ymin+2,#185);
  FOR i:=Ymin+3 TO Ymin+Menulength+2 DO
    BEGIN
      Msg(Xmin,i,#186);Write('':(Xmax-Xmin-1),#186);
    END;
  Msg(Xmin,Ymin+Menulength+3,#204);Msg(Xmax,Ymin+Menulength+3,#185);
  Msg(Xmin,Ymin+Menulength+4,#186);Write('':(Xmax-Xmin-1),#186);
  Msg(Xmin,Ymin+Menulength+5,#200);Msg(Xmax,Ymin+Menulength+5,#188);
  CASE TypeOfConstraints OF
    'N': ToC:='No ';
    'P': ToC:='Pos';
    'M': ToC:='Mon';
    'C': ToC:='Con';
    '0': ToC:=' 0 ';
    '1': ToC:=' 1 ';
    '2': ToC:=' 2 ';
  END;
  IF verbose THEN
    BEGIN
      Box(2,79,23,25,BkColor,TxtColor);
      Box(61,80,Ymin,Ymin+7,BkColor,TxtColor);
      GotoXY(62,Ymin+1);Write(n:4,' B-Splines');
      GotoXY(62,Ymin+2);Write(m:4,' Data values');
      GotoXY(62,Ymin+3);Write(order:4,' Order');
      GotoXY(62,Ymin+4);Write(r:4,'-nd deriv.');
      GotoXY(62,Ymin+5);Write(l:4,' Free knots');
      GotoXY(62,Ymin+6);Write(ToC:4,' Constraints');
    END;
END;           { DrawBorder }

{-----------------------------------------------------------------------------}

PROCEDURE Fill;
VAR j:integer;
BEGIN
  Msg(Xmin+((Xmax-Xmin-2-Length(mm[0,0])) DIV 2),Ymin+1,mm[0,0]);
  FOR j:=1 TO MenuLength DO BEGIN
    Msg(Xmin+4,Ymin+j+2,mm[j,0])
  END;
  Msg(Xmin+2,Ymin+Menulength+4,'ESC    End');
END;  { Fill }

{-----------------------------------------------------------------------------}

FUNCTION ReadMenuNo:char;
CONST arrow='-->';
      clear='   ';
VAR i,iold,err,temp:integer;
    ch:char;
BEGIN
  i:=1;iold:=1;
  REPEAT
    Msg(Xmin+1,iold+Ymin+2,clear);
    Msg(Xmin+1,i+Ymin+2,arrow);
    iold:=i;
    IF verbose THEN
      BEGIN
        GotoXY(3,24);Write(' ':75);
        Msg(40-Length(mm[i,1]) DIV 2,24,mm[i,1]);
      END;
    ReadKbd(ch);
    IF ch=^E THEN Dec(i);
    IF ch=^x THEN Inc(i);
    IF i<1 THEN Inc(i,Menulength);
    IF i>MenuLength THEN Dec(i,MenuLength);
    IF ch=Chr(27) THEN
      BEGIN
        ReadMenuNo:='0';Exit;
      END;
    IF ch=^M THEN
      BEGIN
        ReadMenuNO:=Chr(48+i);Exit;
      END;
  UNTIL FALSE;
END; { ReadMenuNo }

{-----------------------------------------------------------------------------}

BEGIN     { Menu }
  DrawBorder;
  Fill;
  ch:=ReadMenuNo;
  IF ch=#27 THEN Menu:=0 ELSE Menu:=Ord(ch)-48;
END;{menu}

{-----------------------------------------------------------------------------}

END.
